@@ -46,6 +46,9 @@ engine_opts = [
     cfg.IntOpt('check_interval_max',
                default=3600,
                help=_('Maximum seconds between cluster check to be called.')),
+    cfg.IntOpt('health_check_interval_min',
+               default=60,
+               help=_('Minimum seconds between health check to be called.')),
     cfg.IntOpt('periodic_fuzzy_delay',
                default=10,
                help=_('Range of seconds to randomly delay when starting the '
@@ -82,6 +85,16 @@ engine_opts = [
     cfg.IntOpt('lock_retry_interval',
                default=10,
                help=_('Number of seconds between lock retries.')),
+    cfg.IntOpt('database_retry_limit',
+               default=10,
+               help=_('Number of times retrying a failed operation on the '
+                      'database.')),
+    cfg.IntOpt('database_retry_interval',
+               default=0.3,
+               help=_('Initial number of seconds between database retries.')),
+    cfg.IntOpt('database_max_retry_interval',
+               default=2,
+               help=_('Maximum number of seconds between database retries.')),
     cfg.IntOpt('engine_life_check_timeout',
                default=2,
                help=_('RPC timeout for the engine liveness check that is used'
@@ -94,6 +107,12 @@ engine_opts = [
                default=60,
                help=_('Maximum time since last check-in for a service to be '
                       'considered up.')),
+    cfg.IntOpt('scheduler_thread_pool_size',
+               default=1000,
+               help=_('Maximum number of threads to use for scheduler.')),
+    cfg.IntOpt('health_manager_thread_pool_size',
+               default=1000,
+               help=_('Maximum number of threads to use for health manager.')),
 ]
 cfg.CONF.register_opts(engine_opts)
 
@@ -206,6 +225,13 @@ notification_opts = [
 cfg.CONF.register_group(notification_group)
 cfg.CONF.register_opts(notification_opts, group=notification_group)
 
+# Notification topic
+notification_topic_opts = [
+    cfg.ListOpt('notification_topics',
+                default=['versioned_notifications'],
+                help=_('Default notification topic.'))]
+cfg.CONF.register_opts(notification_topic_opts)
+
 # Zaqar group
 zaqar_group = cfg.OptGroup(
     'zaqar', title='Zaqar Options',
