@@ -64,6 +64,8 @@ CLUSTER_ACTION_NAMES = (
     'CLUSTER_OPERATION',
 )
 
+CLUSTER_SCALE_ACTIONS = [CLUSTER_SCALE_IN, CLUSTER_SCALE_OUT]
+
 NODE_ACTION_NAMES = (
     NODE_CREATE, NODE_DELETE, NODE_UPDATE,
     NODE_JOIN, NODE_LEAVE,
@@ -266,6 +268,12 @@ ACTION_STATUSES = (
     'SUSPENDED',
 )
 
+ACTION_PARAMS = (
+    ACTION_UPDATE_FORCE,
+) = (
+    'force',
+)
+
 EVENT_LEVELS = {
     'CRITICAL': logging.CRITICAL,
     'ERROR': logging.ERROR,
@@ -282,12 +290,24 @@ DETECTION_TYPES = (
     # 'LB_STATUS_POLLING',
 )
 
+HEALTH_CHECK_TYPES = (
+    EVENTS, POLLING,
+) = (
+    'EVENTS', 'POLLING'
+)
+
 RECOVERY_ACTIONS = (
     RECOVER_REBOOT, RECOVER_REBUILD, RECOVER_RECREATE,
 ) = (
     'REBOOT', 'REBUILD', 'RECREATE',
 )
 
+RECOVERY_CONDITIONAL = (
+    ALL_FAILED, ANY_FAILED,
+) = (
+    'ALL_FAILED', 'ANY_FAILED',
+)
+
 NOTIFICATION_PRIORITIES = (
     PRIO_AUDIT, PRIO_CRITICAL, PRIO_ERROR, PRIO_WARN, PRIO_INFO, PRIO_DEBUG,
     PRIO_SAMPLE,
@@ -309,6 +329,32 @@ LIFECYCLE_TRANSITION_TYPE = (
 
 VM_STATUS = (
     VS_ACTIVE, VS_ERROR, VS_SUSPENDED, VS_SHUTOFF, VS_PAUSED, VS_RESCUE,
+    VS_DELETED,
 ) = (
-    'ACTIVE', 'ERROR', 'SUSPENDED', 'SHUTOFF', 'PAUSED', 'RESCUE',
+    'ACTIVE', 'ERROR', 'SUSPENDED', 'SHUTOFF', 'PAUSED', 'RESCUE', 'DELETED',
+)
+
+HEALTH_CHECK_MESSAGE = (
+    POLL_STATUS_PASS, POLL_STATUS_FAIL, POLL_URL_PASS, POLL_URL_FAIL,
+) = (
+    'Poll Status health check passed',
+    'Poll Status health check failed',
+    'Poll URL health check passed',
+    'Poll URL health check failed',
+)
+
+CONFLICT_BYPASS_ACTIONS = [
+    CLUSTER_DELETE, NODE_DELETE, NODE_OPERATION,
+]
+
+LOCK_BYPASS_ACTIONS = [
+    CLUSTER_DELETE, NODE_DELETE, NODE_OPERATION,
+]
+
+REBOOT_TYPE = 'type'
+
+REBOOT_TYPES = (
+    REBOOT_SOFT, REBOOT_HARD
+) = (
+    'SOFT', 'HARD'
 )
