@@ -128,6 +128,15 @@ class ResourceInUse(SenlinException):
     msg_fmt = _("The %(type)s '%(id)s' cannot be deleted: %(reason)s.")
 
 
+class ResourceIsLocked(SenlinException):
+    """Generic exception for resource in use.
+
+    The resource type here can be 'cluster', 'node'.
+    """
+    msg_fmt = _("%(action)s for %(type)s '%(id)s' cannot be completed "
+                "because it is already locked.")
+
+
 class ProfileNotSpecified(SenlinException):
     msg_fmt = _("Profile not specified.")
 
@@ -180,6 +189,22 @@ class ActionInProgress(SenlinException):
     msg_fmt = _("The %(type)s '%(id)s' is in status %(status)s.")
 
 
+class ActionConflict(SenlinException):
+    msg_fmt = _("The %(type)s action for target %(target)s conflicts with "
+                "the following action(s): %(actions)s")
+
+
+class ActionCooldown(SenlinException):
+    msg_fmt = _("The %(type)s action for cluster %(cluster)s cannot be "
+                "processed due to Policy %(policy_id)s cooldown still in "
+                "progress")
+
+
+class ActionImmutable(SenlinException):
+    msg_fmt = _("Action (%(id)s) is in status (%(actual)s) while expected "
+                "status must be one of (%(expected)s).")
+
+
 class NodeNotOrphan(SenlinException):
     msg_fmt = _("%(message)s")
 
@@ -188,15 +213,16 @@ class InternalError(SenlinException):
     """A base class for internal exceptions in senlin.
 
     The internal exception classes which inherit from :class:`SenlinException`
-    class should be translated to a user facing exception type if need to be
-    made user visible.
+    class should be translated to a user facing exception type if they need to
+    be made user visible.
     """
     msg_fmt = _("%(message)s")
     message = _('Internal error happened')
 
     def __init__(self, **kwargs):
         self.code = kwargs.pop('code', 500)
-        self.message = kwargs.pop('message', self.message)
+        # If a "message" is not provided, or None or blank, use the default.
+        self.message = kwargs.pop('message', self.message) or self.message
         super(InternalError, self).__init__(
             code=self.code, message=self.message, **kwargs)
 
