@@ -21,13 +21,14 @@ from oslo_utils import timeutils
 from senlin.common import consts
 from senlin.common import context as req_context
 from senlin.common import exception
-from senlin.common.i18n import _
 from senlin.common import utils
 from senlin.engine import dispatcher
 from senlin.engine import event as EVENT
 from senlin.objects import action as ao
+from senlin.objects import cluster_lock as cl
 from senlin.objects import cluster_policy as cpo
 from senlin.objects import dependency as dobj
+from senlin.objects import node_lock as nl
 from senlin.policies import base as policy_mod
 
 wallclock = time.time
@@ -35,7 +36,7 @@ LOG = logging.getLogger(__name__)
 
 
 class Action(object):
-    '''An action can be performed on a cluster or a node of a cluster.'''
+    """An action can be performed on a cluster or a node of a cluster."""
 
     RETURNS = (
         RES_OK, RES_ERROR, RES_RETRY, RES_CANCEL, RES_TIMEOUT,
@@ -208,7 +209,18 @@ class Action(object):
             'data': obj.data,
         }
 
-        return cls(obj.target, obj.action, ctx, **kwargs)
+        target_type = obj.action.split('_')[0]
+        if target_type == 'CLUSTER':
+            from senlin.engine.actions import cluster_action
+            ActionClass = cluster_action.ClusterAction
+        elif target_type == 'NODE':
+            from senlin.engine.actions import node_action
+            ActionClass = node_action.NodeAction
+        else:
+            from senlin.engine.actions import custom_action
+            ActionClass = custom_action.CustomAction
+
+        return ActionClass(obj.target, obj.action, ctx, **kwargs)
 
     @classmethod
     def load(cls, ctx, action_id=None, db_action=None, project_safe=True):
@@ -237,6 +249,10 @@ class Action(object):
         :param dict kwargs: Other keyword arguments for the action.
         :return: ID of the action created.
         """
+
+        cls._check_action_lock(target, action)
+        cls._check_conflicting_actions(ctx, target, action)
+
         params = {
             'user_id': ctx.user_id,
             'project_id': ctx.project_id,
@@ -246,9 +262,37 @@ class Action(object):
             'trusts': ctx.trusts,
         }
         c = req_context.RequestContext.from_dict(params)
+
+        if action in consts.CLUSTER_SCALE_ACTIONS:
+            Action.validate_scaling_action(c, target, action)
+
         obj = cls(target, action, c, **kwargs)
         return obj.store(ctx)
 
+    @staticmethod
+    def _check_action_lock(target, action):
+        if action in consts.LOCK_BYPASS_ACTIONS:
+            return
+        elif (action in list(consts.CLUSTER_ACTION_NAMES) and
+                cl.ClusterLock.is_locked(target)):
+            raise exception.ResourceIsLocked(
+                action=action, type='cluster', id=target)
+        elif (action in list(consts.NODE_ACTION_NAMES) and
+              nl.NodeLock.is_locked(target)):
+            raise exception.ResourceIsLocked(
+                action=action, type='node', id=target)
+
+    @staticmethod
+    def _check_conflicting_actions(ctx, target, action):
+        conflict_actions = ao.Action.get_all_active_by_target(ctx, target)
+        # Ignore conflicting actions on deletes.
+        if not conflict_actions or action in consts.CONFLICT_BYPASS_ACTIONS:
+            return
+        else:
+            action_ids = [a['id'] for a in conflict_actions]
+            raise exception.ActionConflict(
+                type=action, target=target, actions=",".join(action_ids))
+
     @classmethod
     def delete(cls, ctx, action_id):
         """Delete an action from database.
@@ -269,29 +313,99 @@ class Action(object):
             return
 
         if cmd == self.SIG_CANCEL:
-            expected = (self.INIT, self.WAITING, self.READY, self.RUNNING)
+            expected = (self.INIT, self.WAITING, self.READY, self.RUNNING,
+                        self.WAITING_LIFECYCLE_COMPLETION)
         elif cmd == self.SIG_SUSPEND:
             expected = (self.RUNNING)
         else:  # SIG_RESUME
             expected = (self.SUSPENDED)
 
         if self.status not in expected:
-            LOG.error("Action (%(id)s) is in status (%(actual)s) while "
-                      "expected status must be one of (%(expected)s).",
-                      dict(id=self.id[:8], expected=expected,
-                           actual=self.status))
+            LOG.info("Action (%(id)s) is in status (%(actual)s) while "
+                     "expected status must be one of (%(expected)s).",
+                     dict(id=self.id[:8], expected=expected,
+                          actual=self.status))
             return
 
         ao.Action.signal(self.context, self.id, cmd)
 
+    def signal_cancel(self):
+        """Signal the action and any depended actions to cancel.
+
+        If the action or any depended actions are in status
+        'WAITING_LIFECYCLE_COMPLETION' or 'INIT' update the status to cancelled
+         directly.
+
+        :raises: `ActionImmutable` if the action is in an unchangeable state
+        """
+        expected = (self.INIT, self.WAITING, self.READY, self.RUNNING,
+                    self.WAITING_LIFECYCLE_COMPLETION)
+
+        if self.status not in expected:
+            raise exception.ActionImmutable(id=self.id[:8], expected=expected,
+                                            actual=self.status)
+
+        ao.Action.signal(self.context, self.id, self.SIG_CANCEL)
+
+        if self.status in (self.WAITING_LIFECYCLE_COMPLETION, self.INIT):
+            self.set_status(self.RES_CANCEL, 'Action execution cancelled')
+
+        depended = dobj.Dependency.get_depended(self.context, self.id)
+        if not depended:
+            return
+
+        for child in depended:
+            # Try to cancel all dependant actions
+            action = self.load(self.context, action_id=child)
+            if not action.is_cancelled():
+                ao.Action.signal(self.context, child, self.SIG_CANCEL)
+            # If the action is in WAITING_LIFECYCLE_COMPLETION or INIT update
+            # the status to CANCELLED immediately.
+            if action.status in (action.WAITING_LIFECYCLE_COMPLETION,
+                                 action.INIT):
+                action.set_status(action.RES_CANCEL,
+                                  'Action execution cancelled')
+
+    def force_cancel(self):
+        """Force the action and any depended actions to cancel.
+
+        If the action or any depended actions are in status 'INIT', 'WAITING',
+        'READY', 'RUNNING', or 'WAITING_LIFECYCLE_COMPLETION' immediately
+        update their status to cancelled. This should only be used if an action
+        is stuck/dead and has no expectation of ever completing.
+
+        :raises: `ActionImmutable` if the action is in an unchangeable state
+        """
+        expected = (self.INIT, self.WAITING, self.READY, self.RUNNING,
+                    self.WAITING_LIFECYCLE_COMPLETION)
+        if self.status not in expected:
+            raise exception.ActionImmutable(id=self.id[:8], expected=expected,
+                                            actual=self.status)
+        LOG.debug('Forcing action %s to cancel.', self.id)
+        self.set_status(self.RES_CANCEL, 'Action execution force cancelled')
+
+        depended = dobj.Dependency.get_depended(self.context, self.id)
+        if not depended:
+            return
+
+        for child in depended:
+            # Force cancel all dependant actions
+            action = self.load(self.context, action_id=child)
+            if action.status in (action.INIT, action.WAITING, action.READY,
+                                 action.RUNNING,
+                                 action.WAITING_LIFECYCLE_COMPLETION):
+                LOG.debug('Forcing action %s to cancel.', action.id)
+                action.set_status(action.RES_CANCEL,
+                                  'Action execution force cancelled')
+
     def execute(self, **kwargs):
-        '''Execute the action.
+        """Execute the action.
 
         In theory, the action encapsulates all information needed for
         execution.  'kwargs' may specify additional parameters.
         :param kwargs: additional parameters that may override the default
                        properties stored in the action record.
-        '''
+        """
         raise NotImplementedError
 
     def set_status(self, result, reason=None):
@@ -317,10 +431,6 @@ class Action(object):
             status = self.CANCELLED
             ao.Action.mark_cancelled(self.context, self.id, timestamp)
 
-        elif result == self.RES_LIFECYCLE_COMPLETE:
-            status = self.SUCCEEDED
-            ao.Action.mark_ready(self.context, self.id, timestamp)
-
         else:  # result == self.RES_RETRY:
             retries = self.data.get('retries', 0)
             # Action failed at the moment, but can be retried
@@ -336,8 +446,9 @@ class Action(object):
                 dispatcher.start_action(self.id)
             else:
                 status = self.RES_ERROR
-                reason = reason or _('Exceeded maximum number of retries (%d)'
-                                     ) % cfg.CONF.lock_retry_times
+                if not reason:
+                    reason = ('Exceeded maximum number of retries (%d)'
+                              '') % cfg.CONF.lock_retry_times
                 ao.Action.mark_failed(self.context, self.id, timestamp, reason)
 
         if status == self.SUCCEEDED:
@@ -359,6 +470,8 @@ class Action(object):
     def is_timeout(self, timeout=None):
         if timeout is None:
             timeout = self.timeout
+        if self.start_time is None:
+            return False
         time_elapse = wallclock() - self.start_time
         return time_elapse > timeout
 
@@ -381,7 +494,7 @@ class Action(object):
         return self._check_signal() == self.SIG_RESUME
 
     def _check_result(self, name):
-        """Check policy check status and generate event.
+        """Check policy status and generate event.
 
         :param name: Name of policy checked
         :return: True if the policy checking can be continued, or False if the
@@ -391,8 +504,8 @@ class Action(object):
         if self.data['status'] == policy_mod.CHECK_OK:
             return True
 
-        self.data['reason'] = _("Failed policy '%(name)s': %(reason)s"
-                                ) % {'name': name, 'reason': reason}
+        self.data['reason'] = ("Failed policy '%(name)s': %(reason)s"
+                               ) % {'name': name, 'reason': reason}
         return False
 
     def policy_check(self, cluster_id, target):
@@ -412,18 +525,14 @@ class Action(object):
                                              filters={'enabled': True})
         # default values
         self.data['status'] = policy_mod.CHECK_OK
-        self.data['reason'] = _('Completed policy checking.')
+        self.data['reason'] = 'Completed policy checking.'
 
         for pb in bindings:
             policy = policy_mod.Policy.load(self.context, pb.policy_id)
-            # We record the last operation time for all policies bound to the
-            # cluster, no matter that policy is only interested in the
-            # "BEFORE" or "AFTER" or both.
-            if target == 'AFTER':
-                ts = timeutils.utcnow(True)
-                pb.last_op = ts
-                cpo.ClusterPolicy.update(self.context, pb.cluster_id,
-                                         pb.policy_id, {'last_op': ts})
+
+            # add last_op as input for the policy so that it can be used
+            # during pre_op
+            self.inputs['last_op'] = pb.last_op
 
             if not policy.need_check(target, self):
                 continue
@@ -433,13 +542,6 @@ class Action(object):
             else:  # target == 'AFTER'
                 method = getattr(policy, 'post_op', None)
 
-            if getattr(policy, 'cooldown', None):
-                if pb.cooldown_inprogress(policy.cooldown):
-                    self.data['status'] = policy_mod.CHECK_ERROR
-                    self.data['reason'] = _('Policy %s cooldown is still '
-                                            'in progress.') % policy.id
-                    return
-
             if method is not None:
                 method(cluster_id, self)
 
@@ -448,6 +550,70 @@ class Action(object):
                 return
         return
 
+    @staticmethod
+    def validate_scaling_action(ctx, cluster_id, action):
+        """Validate scaling action against actions table and policy cooldown.
+
+        :param ctx: An instance of the request context.
+        :param cluster_id: ID of the cluster the scaling action is targeting.
+        :param action: Scaling action being validated.
+        :return: None
+        :raises: An exception of ``ActionCooldown`` when the action being
+        validated is still in cooldown based off the policy or
+        ``ActionConflict`` when a scaling action is already in the action
+        table.
+        """
+        # Check for conflicting actions in the actions table.
+        conflicting_actions = Action._get_conflicting_scaling_actions(
+            ctx, cluster_id)
+        if conflicting_actions:
+            action_ids = [a.get('id', None) for a in conflicting_actions]
+            LOG.info("Unable to process %(action)s for cluster %(cluster_id)s "
+                     "the action conflicts with %(conflicts)s",
+                     {'action': action,
+                      'cluster_id': cluster_id,
+                      'conflicts': action_ids})
+            raise exception.ActionConflict(
+                type=action,
+                target=cluster_id,
+                actions=",".join(action_ids))
+
+        # Check to see if action cooldown should be observed.
+        bindings = cpo.ClusterPolicy.get_all(ctx, cluster_id,
+                                             sort='priority',
+                                             filters={'enabled': True})
+        for pb in bindings:
+            policy = policy_mod.Policy.load(ctx, pb.policy_id)
+            if getattr(policy, 'cooldown', None) and policy.event == action:
+                if pb.last_op and not timeutils.is_older_than(
+                        pb.last_op, policy.cooldown):
+                    LOG.info("Unable to process %(action)s for cluster "
+                             "%(cluster_id)s the actions policy %(policy)s "
+                             "cooldown still in progress",
+                             {'action': action,
+                              'cluster_id': cluster_id,
+                              'policy': pb.policy_id})
+                    raise exception.ActionCooldown(
+                        type=action,
+                        cluster=cluster_id,
+                        policy_id=pb.policy_id)
+        return
+
+    @staticmethod
+    def _get_conflicting_scaling_actions(ctx, cluster_id):
+        """Check actions table for conflicting scaling actions.
+
+        :param ctx: An instance of the request context.
+        :param cluster_id: ID of the cluster the scaling action is targeting.
+        :return: A list of conflicting actions.
+        """
+        scaling_actions = ao.Action.action_list_active_scaling(
+            ctx, cluster_id)
+        if scaling_actions:
+            return [a.to_dict() for a in scaling_actions]
+        else:
+            return None
+
     def to_dict(self):
         if self.id:
             dep_on = dobj.Dependency.get_depended(self.context, self.id)
@@ -482,7 +648,7 @@ class Action(object):
 
 
 def ActionProc(ctx, action_id):
-    '''Action process.'''
+    """Action process."""
 
     # Step 1: materialize the action object
     action = Action.load(ctx, action_id=action_id, project_safe=False)
@@ -490,6 +656,13 @@ def ActionProc(ctx, action_id):
         LOG.error('Action "%s" could not be found.', action_id)
         return False
 
+    if action.is_cancelled():
+        reason = '%(action)s [%(id)s] cancelled' % {
+            'action': action.action, 'id': action.id[:8]}
+        action.set_status(action.RES_CANCEL, reason)
+        LOG.info(reason)
+        return True
+
     EVENT.info(action, consts.PHASE_START, action_id[:8])
 
     reason = 'Action completed'
