@@ -68,23 +68,33 @@ class ClusterAction(base.Action):
         status = self.get_status()
         while status != self.READY:
             if status == self.FAILED:
-                reason = ('%(action)s [%(id)s] failed') % {
-                    'action': self.action, 'id': self.id[:8]}
+                reason = ('%(action)s [%(id)s] failed' % {
+                    'action': self.action, 'id': self.id[:8]})
                 LOG.debug(reason)
                 return self.RES_ERROR, reason
 
             if self.is_cancelled():
                 # During this period, if cancel request comes, cancel this
-                # operation immediately, then release the cluster lock
-                reason = ('%(action)s [%(id)s] cancelled') % {
-                    'action': self.action, 'id': self.id[:8]}
+                # operation immediately after signaling children to cancel,
+                # then release the cluster lock
+                reason = ('%(action)s [%(id)s] cancelled' % {
+                    'action': self.action, 'id': self.id[:8]})
                 LOG.debug(reason)
                 return self.RES_CANCEL, reason
 
+            # When a child action is cancelled the parent action will update
+            # its status to cancelled as well this allows it to exit.
+            if status == self.CANCELLED:
+                if self.check_children_complete():
+                    reason = ('%(action)s [%(id)s] cancelled' % {
+                        'action': self.action, 'id': self.id[:8]})
+                    LOG.debug(reason)
+                    return self.RES_CANCEL, reason
+
             if self.is_timeout():
                 # Action timeout, return
-                reason = ('%(action)s [%(id)s] timeout') % {
-                    'action': self.action, 'id': self.id[:8]}
+                reason = ('%(action)s [%(id)s] timeout' % {
+                    'action': self.action, 'id': self.id[:8]})
                 LOG.debug(reason)
                 return self.RES_TIMEOUT, reason
 
@@ -100,9 +110,23 @@ class ClusterAction(base.Action):
             # Continue waiting (with reschedule)
             scheduler.reschedule(self.id, 3)
             status = self.get_status()
+            dispatcher.start_action()
 
         return self.RES_OK, 'All dependents ended with success'
 
+    def check_children_complete(self):
+        depended = dobj.Dependency.get_depended(self.context, self.id)
+        if not depended:
+            return True
+
+        for child in depended:
+            # Try to cancel all dependant actions
+            action = base.Action.load(self.context, action_id=child)
+            if action.get_status() not in (action.CANCELLED, action.SUCCEEDED,
+                                           action.FAILED):
+                return False
+        return True
+
     def _create_nodes(self, count):
         """Utility method for node creation.
 
@@ -302,8 +326,11 @@ class ClusterAction(base.Action):
         for action_id, node_id in child:
             status = ao.Action.check_status(self.context, action_id, 0)
             if (status == consts.ACTION_WAITING_LIFECYCLE_COMPLETION):
+                # update action status and reset owner back to None
+                # so that the action will get picked up by dispatcher
                 ao.Action.update(self.context, action_id,
-                                 {'status': base.Action.READY})
+                                 {'status': base.Action.READY,
+                                  'owner': None})
 
     def _remove_nodes_with_hook(self, action_name, node_ids, lifecycle_hook,
                                 inputs=None):
@@ -341,9 +368,11 @@ class ClusterAction(base.Action):
 
             notifier = msg.Message(lifecycle_hook_target, **kwargs)
 
-            for action_id, node_id in child:
+            child_copy = list(child)
+            for action_id, node_id in child_copy:
                 # wait lifecycle complete if node exists and is active
                 node = no.Node.get(self.context, node_id)
+                owner = None
                 if not node:
                     LOG.warning('Node %s is not found. '
                                 'Skipping wait for lifecycle completion.',
@@ -358,9 +387,14 @@ class ClusterAction(base.Action):
                     child.remove((action_id, node_id))
                 else:
                     status = base.Action.WAITING_LIFECYCLE_COMPLETION
+                    # set owner for actions in waiting for lifecycle completion
+                    # so that they will get cleaned up by dead engine gc
+                    # if the engine dies
+                    owner = self.owner
 
                 ao.Action.update(self.context, action_id,
-                                 {'status': status})
+                                 {'status': status,
+                                  'owner': owner})
                 if status == base.Action.WAITING_LIFECYCLE_COMPLETION:
                     notifier.post_lifecycle_hook_message(
                         action_id, node_id, node.physical_id,
@@ -422,9 +456,11 @@ class ClusterAction(base.Action):
         lifecycle_hook = self.data.get('hooks')
         if lifecycle_hook:
             if stop_node_before_delete:
+                # set update_parent_status to False so that a failure in stop
+                # operation is ignored and the parent status is not changed
                 res, reason = self._remove_nodes_with_hook(
                     consts.NODE_OPERATION, node_ids, lifecycle_hook,
-                    {'operation': 'stop'})
+                    {'operation': 'stop', 'update_parent_status': False})
                 if res != self.RES_OK:
                     LOG.warning('Failure while stopping nodes. '
                                 'Proceed to delete nodes.')
@@ -435,8 +471,11 @@ class ClusterAction(base.Action):
                     action_name, node_ids, lifecycle_hook)
         else:
             if stop_node_before_delete:
+                # set update_parent_status to False so that a failure in stop
+                # operation is ignored and the parent status is not changed
                 res, reason = self._remove_nodes_normally(
-                    consts.NODE_OPERATION, node_ids, {'operation': 'stop'})
+                    consts.NODE_OPERATION, node_ids,
+                    {'operation': 'stop', 'update_parent_status': False})
                 if res != self.RES_OK:
                     LOG.warning('Failure while stopping nodes. '
                                 'Proceed to delete nodes.')
@@ -795,22 +834,11 @@ class ClusterAction(base.Action):
         """
         self.entity.do_recover(self.context)
 
-        # process data from health_policy
-        pd = self.data.get('health', None)
         inputs = {}
-        if pd:
-            check = self.data.get('check', False)
-            recover_action = pd.get('recover_action', None)
-            fencing = pd.get('fencing', None)
-            if recover_action is not None:
-                inputs['operation'] = recover_action
-            if fencing is not None and 'COMPUTE' in fencing:
-                inputs['params'] = {'fence_compute': True}
-        else:
-            check = self.inputs.get('check', False)
-            recover_action = self.inputs.get('operation', None)
-            if recover_action is not None:
-                inputs['operation'] = recover_action
+
+        check = self.inputs.get('check', False)
+        inputs['operation'] = self.inputs.get('operation', None)
+        inputs['operation_params'] = self.inputs.get('operation_params', None)
 
         children = []
         for node in self.entity.nodes:
@@ -1128,11 +1156,11 @@ class ClusterAction(base.Action):
         result, reason = method()
 
         # do post-action policy checking
-        if result == self.RES_OK:
-            self.policy_check(self.entity.id, 'AFTER')
-            if self.data['status'] != policy_mod.CHECK_OK:
-                reason = 'Policy check failure: %s' % self.data['reason']
-                return self.RES_ERROR, reason
+        self.inputs['action_result'] = result
+        self.policy_check(self.entity.id, 'AFTER')
+        if self.data['status'] != policy_mod.CHECK_OK:
+            reason = 'Policy check failure: %s' % self.data['reason']
+            return self.RES_ERROR, reason
 
         return result, reason
 
@@ -1146,7 +1174,7 @@ class ClusterAction(base.Action):
                  was a success and why if it wasn't a success.
         """
         # Try to lock cluster before do real operation
-        forced = True if self.action == consts.CLUSTER_DELETE else False
+        forced = (self.action == consts.CLUSTER_DELETE)
         res = senlin_lock.cluster_lock_acquire(self.context, self.target,
                                                self.id, self.owner,
                                                senlin_lock.CLUSTER_SCOPE,
@@ -1179,7 +1207,12 @@ def CompleteLifecycleProc(context, action_id):
         raise exception.ResourceNotFound(type='action', id=action_id)
 
     if action.get_status() == consts.ACTION_WAITING_LIFECYCLE_COMPLETION:
-        action.set_status(base.Action.RES_LIFECYCLE_COMPLETE)
+        # update action status and reset owner back to None
+        # so that the action will get picked up by dispatcher
+        ao.Action.update(context, action_id,
+                         {'status': consts.ACTION_READY,
+                          'status_reason': 'Lifecycle complete.',
+                          'owner': None})
         dispatcher.start_action()
     else:
         LOG.debug('Action %s status is not WAITING_LIFECYCLE.  '
