@@ -67,7 +67,7 @@ class NodeAction(base.Action):
                                {'cluster_id': '', 'status': consts.NS_ERROR})
                 return self.RES_ERROR, result
 
-        res = self.entity.do_create(self.context)
+        res, reason = self.entity.do_create(self.context)
 
         if cluster_id and self.cause == consts.CAUSE_RPC:
             # Update cluster's desired_capacity and re-evaluate its status no
@@ -79,7 +79,7 @@ class NodeAction(base.Action):
         if res:
             return self.RES_OK, 'Node created successfully.'
         else:
-            return self.RES_ERROR, 'Node creation failed.'
+            return self.RES_ERROR, reason
 
     @profiler.trace('NodeAction.do_delete', hide_args=False)
     def do_delete(self):
@@ -247,6 +247,7 @@ class NodeAction(base.Action):
         """
         # Since node.cluster_id could be reset to '' during action execution,
         # we record it here for policy check and cluster lock release.
+        forced = (self.action in [consts.NODE_DELETE, consts.NODE_OPERATION])
         saved_cluster_id = self.entity.cluster_id
         if saved_cluster_id:
             if self.cause == consts.CAUSE_RPC:
@@ -257,33 +258,32 @@ class NodeAction(base.Action):
                 if not res:
                     return self.RES_RETRY, 'Failed in locking cluster'
 
-                self.policy_check(self.entity.cluster_id, 'BEFORE')
-                if self.data['status'] != pb.CHECK_OK:
-                    # Don't emit message since policy_check should have done it
-                    senlin_lock.cluster_lock_release(saved_cluster_id, self.id,
-                                                     senlin_lock.NODE_SCOPE)
-                    return self.RES_ERROR, ('Policy check: ' +
-                                            self.data['reason'])
+                try:
+                    self.policy_check(self.entity.cluster_id, 'BEFORE')
+                finally:
+                    if self.data['status'] != pb.CHECK_OK:
+                        # Don't emit message since policy_check should have
+                        # done it
+                        senlin_lock.cluster_lock_release(
+                            saved_cluster_id, self.id, senlin_lock.NODE_SCOPE)
+                        return self.RES_ERROR, ('Policy check: ' +
+                                                self.data['reason'])
             elif self.cause == consts.CAUSE_DERIVED_LCH:
-                self.policy_check(self.entity.cluster_id, 'BEFORE')
+                self.policy_check(saved_cluster_id, 'BEFORE')
 
-        reason = ''
         try:
             res = senlin_lock.node_lock_acquire(self.context, self.entity.id,
-                                                self.id, self.owner, False)
+                                                self.id, self.owner, forced)
             if not res:
                 res = self.RES_RETRY
                 reason = 'Failed in locking node'
             else:
                 res, reason = self._execute()
-                if (res == self.RES_OK and saved_cluster_id and
-                        self.cause == consts.CAUSE_RPC):
+                if saved_cluster_id and self.cause == consts.CAUSE_RPC:
                     self.policy_check(saved_cluster_id, 'AFTER')
                     if self.data['status'] != pb.CHECK_OK:
                         res = self.RES_ERROR
                         reason = 'Policy check: ' + self.data['reason']
-                    else:
-                        res = self.RES_OK
         finally:
             senlin_lock.node_lock_release(self.entity.id, self.id)
             if saved_cluster_id and self.cause == consts.CAUSE_RPC:
