@@ -155,7 +155,7 @@ class Node(object):
 
     @classmethod
     def load(cls, context, node_id=None, db_node=None, project_safe=True):
-        '''Retrieve a node from database.'''
+        """Retrieve a node from database."""
         if db_node is None:
             db_node = no.Node.get(context, node_id, project_safe=project_safe)
             if db_node is None:
@@ -166,7 +166,7 @@ class Node(object):
     @classmethod
     def load_all(cls, context, cluster_id=None, limit=None, marker=None,
                  sort=None, filters=None, project_safe=True):
-        '''Retrieve all nodes of from database.'''
+        """Retrieve all nodes of from database."""
         objs = no.Node.get_all(context, cluster_id=cluster_id,
                                filters=filters, sort=sort,
                                limit=limit, marker=marker,
@@ -211,7 +211,9 @@ class Node(object):
     def do_create(self, context):
         if self.status != consts.NS_INIT:
             LOG.error('Node is in status "%s"', self.status)
-            return False
+            self.set_status(context, consts.NS_ERROR,
+                            'Node must be in INIT status')
+            return False, 'Node must be in INIT status'
 
         self.set_status(context, consts.NS_CREATING, 'Creation in progress')
         try:
@@ -220,11 +222,11 @@ class Node(object):
             physical_id = ex.resource_id
             self.set_status(context, consts.NS_ERROR, six.text_type(ex),
                             physical_id=physical_id)
-            return False
+            return False, str(ex)
 
         self.set_status(context, consts.NS_ACTIVE, 'Creation succeeded',
                         physical_id=physical_id)
-        return True
+        return True, None
 
     def do_delete(self, context):
         self.set_status(context, consts.NS_DELETING, 'Deletion in progress')
@@ -280,39 +282,40 @@ class Node(object):
         if self.cluster_id == cluster_id:
             return True
 
+        try:
+            res = pb.Profile.join_cluster(context, self, cluster_id)
+        except exc.EResourceUpdate as ex:
+            LOG.error('Node join cluster faild: %s.', ex)
+            return False
+
+        if not res:
+            return False
         timestamp = timeutils.utcnow(True)
         db_node = no.Node.migrate(context, self.id, cluster_id, timestamp)
         self.cluster_id = cluster_id
         self.updated_at = timestamp
         self.index = db_node.index
-
-        res = pb.Profile.join_cluster(context, self, cluster_id)
-        if res:
-            return True
-
-        # rollback
-        db_node = no.Node.migrate(context, self.id, None, timestamp)
-        self.cluster_id = ''
-        self.updated_at = timestamp
-        self.index = -1
-
-        return False
+        return True
 
     def do_leave(self, context):
         if self.cluster_id == '':
             return True
 
-        res = pb.Profile.leave_cluster(context, self)
-        if res:
-            timestamp = timeutils.utcnow(True)
-            no.Node.migrate(context, self.id, None, timestamp)
-            self.cluster_id = ''
-            self.updated_at = timestamp
-            self.index = -1
-            return True
-        else:
+        try:
+            res = pb.Profile.leave_cluster(context, self)
+        except exc.EResourceDeletion as ex:
+            LOG.error('Node leave cluster faild: %s.', ex)
             return False
 
+        if not res:
+            return False
+        timestamp = timeutils.utcnow(True)
+        no.Node.migrate(context, self.id, None, timestamp)
+        self.cluster_id = ''
+        self.updated_at = timestamp
+        self.index = -1
+        return True
+
     def do_check(self, context):
         if not self.physical_id:
             return False
@@ -344,6 +347,17 @@ class Node(object):
 
         return True
 
+    def do_healthcheck(self, context):
+        """health check a node.
+
+        This function is supposed to be invoked from the health manager to
+        check the health of a given node
+        :param context: The request context of the action.
+        :returns: True if node is healthy. False otherwise.
+        """
+
+        return pb.Profile.healthcheck_object(context, self)
+
     def do_recover(self, context, action):
         """recover a node.
 
@@ -352,10 +366,18 @@ class Node(object):
         :param dict options: A map containing the recovery actions (with
             parameters if any) and fencing settings.
         """
-        if not self.physical_id:
+        options = action.inputs
+
+        operation = options.get('operation', None)
+
+        if (not self.physical_id and operation and
+                (operation.upper() == consts.RECOVER_REBOOT or
+                 operation.upper() == consts.RECOVER_REBUILD)):
+            # physical id is required for REBOOT or REBUILD operations
+            LOG.warning('Recovery failed because node has no physical id'
+                        ' was provided for reboot or rebuild operation.')
             return False
 
-        options = action.inputs
         if options.get('check', False):
             res = False
             try:
@@ -372,17 +394,18 @@ class Node(object):
                         reason='Recovery in progress')
 
         try:
-            physical_id = pb.Profile.recover_object(context, self, **options)
+            physical_id, status = pb.Profile.recover_object(context,
+                                                            self, **options)
         except exc.EResourceOperation as ex:
             self.set_status(context, consts.NS_ERROR, reason=six.text_type(ex))
             return False
 
-        if not physical_id:
+        if not status:
             self.set_status(context, consts.NS_ERROR, reason='Recovery failed')
             return False
 
         params = {}
-        if self.physical_id != physical_id:
+        if physical_id and self.physical_id != physical_id:
             self.data['recovery'] = consts.RECOVER_RECREATE
             params['data'] = self.data
             params['physical_id'] = physical_id
@@ -408,9 +431,10 @@ class Node(object):
 
         try:
             profile = self.rt['profile']
-            method = getattr(profile, 'handle_' + op)
+            method = getattr(profile, 'handle_%s' % op)
             method(self, **params)
         except exc.EResourceOperation as ex:
+            LOG.error('Node operation %s failed: %s.', op, ex)
             self.set_status(context, consts.NS_ERROR, reason=six.text_type(ex))
             return False
 
