@@ -14,7 +14,7 @@
 Policy for load-balancing among nodes in a cluster.
 
 NOTE: For full documentation about how the load-balancing policy works,
-check: https://docs.openstack.org/senlin/latest/developer/policies/
+check: https://docs.openstack.org/senlin/latest/contributor/policies/
 load_balance_v1.html
 """
 
@@ -395,7 +395,7 @@ class LoadBalancingPolicy(base.Policy):
         :returns: When the operation was successful, returns a tuple of
             (True, data) where the data contains references to the resources
             created; otherwise returns a tuple of (False, err) where the err
-            contains a error message.
+            contains an error message.
         """
         reason = _('LB resources deletion succeeded.')
         lb_driver = self.lbaas(cluster.user, cluster.project)
@@ -550,7 +550,7 @@ class LoadBalancingPolicy(base.Policy):
 
     def _get_post_candidates(self, action):
         # This method will parse action data passed from action layer
-        candidates = []
+
         if (action.action == consts.NODE_CREATE or
                 action.action == consts.NODE_RECOVER):
             candidates = [action.entity.id]
@@ -571,7 +571,7 @@ class LoadBalancingPolicy(base.Policy):
         # lb_member is None, need to add to lb pool
         if not lb_member:
             values['data'] = data
-            no.Node.update(action.context, values)
+            no.Node.update(action.context, node.id, values)
             return candidates
 
         # was a member of lb pool, check whether has been recreated
@@ -580,7 +580,7 @@ class LoadBalancingPolicy(base.Policy):
                                 handle_err=False)
             data.pop('lb_member', None)
             values['data'] = data
-            no.Node.update(action.context, values)
+            no.Node.update(action.context, node.id, values)
             return candidates
 
         return None
@@ -637,6 +637,11 @@ class LoadBalancingPolicy(base.Policy):
         :returns: Nothing.
         """
 
+        # skip post op if action did not complete successfully
+        action_result = action.inputs.get('action_result', 'OK')
+        if action_result != 'OK':
+            return
+
         # TODO(Yanyanhu): Need special handling for cross-az scenario
         # which is supported by Neutron lbaas.
         candidates = self._get_post_candidates(action)
