@@ -29,8 +29,10 @@ from senlin.engine import environment
 from senlin.engine import event as EVENT
 from senlin.engine import node as node_mod
 from senlin.objects import action as ao
+from senlin.objects import cluster_lock as cl
 from senlin.objects import cluster_policy as cpo
 from senlin.objects import dependency as dobj
+from senlin.objects import node_lock as nl
 from senlin.policies import base as policy_mod
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
@@ -42,6 +44,8 @@ OWNER_ID = 'c7114713-ee68-409d-ba5d-0560a72a386c'
 ACTION_ID = '4c2cead2-fd74-418a-9d12-bd2d9bd7a812'
 USER_ID = '3c4d64baadcd437d8dd49054899e73dd'
 PROJECT_ID = 'cf7a6ae28dde4f46aa8fe55d318a608f'
+CHILD_IDS = ['8500ae8f-e632-4e8b-8206-552873cc2c3a',
+             '67c0eba9-514f-4659-9deb-99868873dfd6']
 
 
 class DummyAction(ab.Action):
@@ -92,6 +96,11 @@ class ActionBaseTest(base.SenlinTestCase):
         self.assertIsNone(obj.updated_at)
         self.assertEqual({}, obj.data)
 
+    def _create_cp_binding(self, cluster_id, policy_id):
+        return cpo.ClusterPolicy(cluster_id=cluster_id, policy_id=policy_id,
+                                 enabled=True, id=uuidutils.generate_uuid(),
+                                 last_op=None)
+
     @mock.patch.object(cluster_mod.Cluster, 'load')
     def test_action_new_cluster(self, mock_load):
         fake_cluster = mock.Mock(timeout=cfg.CONF.default_action_timeout)
@@ -245,6 +254,191 @@ class ActionBaseTest(base.SenlinTestCase):
         self.assertEqual('FAKE_ID', result)
         mock_store.assert_called_once_with(self.ctx)
 
+    @mock.patch.object(ab.Action, 'store')
+    @mock.patch.object(ao.Action, 'get_all_active_by_target')
+    @mock.patch.object(cl.ClusterLock, 'is_locked')
+    def test_action_create_lock_cluster_false(self, mock_lock,
+                                              mock_active, mock_store):
+        mock_store.return_value = 'FAKE_ID'
+        mock_active.return_value = None
+        mock_lock.return_value = False
+
+        result = ab.Action.create(self.ctx, OBJID, 'CLUSTER_CREATE',
+                                  name='test')
+
+        self.assertEqual('FAKE_ID', result)
+        mock_store.assert_called_once_with(self.ctx)
+        mock_active.assert_called_once_with(mock.ANY, OBJID)
+
+    @mock.patch.object(ab.Action, 'store')
+    @mock.patch.object(ao.Action, 'get_all_active_by_target')
+    @mock.patch.object(cl.ClusterLock, 'is_locked')
+    def test_action_create_lock_cluster_true(self, mock_lock,
+                                             mock_active, mock_store):
+        mock_store.return_value = 'FAKE_ID'
+        mock_active.return_value = None
+        mock_lock.return_value = True
+
+        error_message = (
+            'CLUSTER_CREATE for cluster \'{}\' cannot be completed because '
+            'it is already locked.').format(OBJID)
+        with self.assertRaisesRegexp(exception.ResourceIsLocked,
+                                     error_message):
+            ab.Action.create(self.ctx, OBJID, 'CLUSTER_CREATE', name='test')
+
+        mock_store.assert_not_called()
+        mock_active.assert_not_called()
+
+    @mock.patch.object(ab.Action, 'store')
+    @mock.patch.object(ao.Action, 'get_all_active_by_target')
+    @mock.patch.object(nl.NodeLock, 'is_locked')
+    def test_action_create_lock_node_false(self, mock_lock,
+                                           mock_active, mock_store):
+        mock_store.return_value = 'FAKE_ID'
+        mock_active.return_value = None
+        mock_lock.return_value = False
+
+        result = ab.Action.create(self.ctx, OBJID, 'NODE_CREATE',
+                                  name='test')
+
+        self.assertEqual('FAKE_ID', result)
+        mock_store.assert_called_once_with(self.ctx)
+        mock_active.assert_called_once_with(mock.ANY, OBJID)
+
+    @mock.patch.object(ab.Action, 'store')
+    @mock.patch.object(ao.Action, 'get_all_active_by_target')
+    @mock.patch.object(cl.ClusterLock, 'is_locked')
+    def test_action_create_lock_cluster_true_delete(self, mock_lock,
+                                                    mock_active, mock_store):
+        mock_store.return_value = 'FAKE_ID'
+        mock_active.return_value = None
+        mock_lock.return_value = True
+
+        result = ab.Action.create(self.ctx, OBJID, 'CLUSTER_DELETE',
+                                  name='test')
+
+        self.assertEqual('FAKE_ID', result)
+        mock_store.assert_called_once_with(self.ctx)
+        mock_active.assert_called_once_with(mock.ANY, OBJID)
+
+    @mock.patch.object(ab.Action, 'store')
+    @mock.patch.object(ao.Action, 'get_all_active_by_target')
+    @mock.patch.object(nl.NodeLock, 'is_locked')
+    def test_action_create_lock_node_true(self, mock_lock, mock_active,
+                                          mock_store):
+        mock_store.return_value = 'FAKE_ID'
+        mock_active.return_value = None
+        mock_lock.return_value = True
+
+        error_message = (
+            'NODE_CREATE for node \'{}\' cannot be completed because '
+            'it is already locked.').format(OBJID)
+        with self.assertRaisesRegexp(exception.ResourceIsLocked,
+                                     error_message):
+            ab.Action.create(self.ctx, OBJID, 'NODE_CREATE', name='test')
+
+        mock_store.assert_not_called()
+        mock_active.assert_not_called()
+
+    @mock.patch.object(ab.Action, 'store')
+    @mock.patch.object(ao.Action, 'get_all_active_by_target')
+    @mock.patch.object(cl.ClusterLock, 'is_locked')
+    def test_action_create_conflict(self, mock_lock, mock_active, mock_store):
+        mock_store.return_value = 'FAKE_ID'
+        uuid1 = 'ce982cd5-26da-4e2c-84e5-be8f720b7478'
+        uuid2 = 'ce982cd5-26da-4e2c-84e5-be8f720b7479'
+        mock_active.return_value = [ao.Action(id=uuid1), ao.Action(id=uuid2)]
+        mock_lock.return_value = False
+
+        error_message = (
+            'The NODE_CREATE action for target {} conflicts with the following'
+            ' action\(s\): {},{}').format(OBJID, uuid1, uuid2)
+        with self.assertRaisesRegexp(exception.ActionConflict,
+                                     error_message):
+            ab.Action.create(self.ctx, OBJID, 'NODE_CREATE', name='test')
+
+        mock_store.assert_not_called()
+        mock_active.assert_called_once_with(mock.ANY, OBJID)
+
+    @mock.patch.object(ab.Action, 'store')
+    @mock.patch.object(ao.Action, 'get_all_active_by_target')
+    @mock.patch.object(cl.ClusterLock, 'is_locked')
+    def test_action_create_delete_no_conflict(self, mock_lock, mock_active,
+                                              mock_store):
+        mock_store.return_value = 'FAKE_ID'
+        uuid1 = 'ce982cd5-26da-4e2c-84e5-be8f720b7478'
+        uuid2 = 'ce982cd5-26da-4e2c-84e5-be8f720b7479'
+        mock_active.return_value = [
+            ao.Action(id=uuid1, action='NODE_DELETE'),
+            ao.Action(id=uuid2, action='NODE_DELETE')
+        ]
+        mock_lock.return_value = True
+
+        result = ab.Action.create(self.ctx, OBJID, 'CLUSTER_DELETE',
+                                  name='test')
+
+        self.assertEqual('FAKE_ID', result)
+        mock_store.assert_called_once_with(self.ctx)
+        mock_active.assert_called_once_with(mock.ANY, OBJID)
+
+    @mock.patch.object(ab.Action, 'store')
+    @mock.patch.object(ao.Action, 'get_all_active_by_target')
+    @mock.patch.object(cl.ClusterLock, 'is_locked')
+    def test_action_create_node_operation_no_conflict(self, mock_lock,
+                                                      mock_active, mock_store):
+        mock_store.return_value = 'FAKE_ID'
+        uuid1 = 'ce982cd5-26da-4e2c-84e5-be8f720b7478'
+        uuid2 = 'ce982cd5-26da-4e2c-84e5-be8f720b7479'
+        mock_active.return_value = [
+            ao.Action(id=uuid1, action='NODE_DELETE'),
+            ao.Action(id=uuid2, action='NODE_DELETE')
+        ]
+        mock_lock.return_value = True
+
+        result = ab.Action.create(self.ctx, OBJID, 'NODE_OPERATION',
+                                  name='test')
+
+        self.assertEqual('FAKE_ID', result)
+        mock_store.assert_called_once_with(self.ctx)
+        mock_active.assert_called_once_with(mock.ANY, OBJID)
+
+    @mock.patch.object(timeutils, 'is_older_than')
+    @mock.patch.object(cpo.ClusterPolicy, 'get_all')
+    @mock.patch.object(policy_mod.Policy, 'load')
+    @mock.patch.object(ab.Action, 'store')
+    def test_action_create_scaling_cooldown_in_progress(self, mock_store,
+                                                        mock_load,
+                                                        mock_load_all,
+                                                        mock_time_util):
+        cluster_id = CLUSTER_ID
+        # Note: policy is mocked
+        policy_id = uuidutils.generate_uuid()
+        policy = mock.Mock(id=policy_id,
+                           TARGET=[('AFTER', 'CLUSTER_SCALE_OUT')],
+                           event='CLUSTER_SCALE_OUT',
+                           cooldown=240)
+        pb = self._create_cp_binding(cluster_id, policy.id)
+        pb.last_op = timeutils.utcnow(True)
+        mock_load_all.return_value = [pb]
+        mock_load.return_value = policy
+        mock_time_util.return_value = False
+        self.assertRaises(exception.ActionCooldown, ab.Action.create, self.ctx,
+                          cluster_id, 'CLUSTER_SCALE_OUT')
+        self.assertEqual(0, mock_store.call_count)
+
+    @mock.patch.object(ao.Action, 'action_list_active_scaling')
+    @mock.patch.object(ab.Action, 'store')
+    def test_action_create_scaling_conflict(self, mock_store,
+                                            mock_list_active):
+        cluster_id = CLUSTER_ID
+
+        mock_action = mock.Mock()
+        mock_action.to_dict.return_value = {'id': 'fake_action_id'}
+        mock_list_active.return_value = [mock_action]
+        self.assertRaises(exception.ActionConflict, ab.Action.create, self.ctx,
+                          cluster_id, 'CLUSTER_SCALE_IN')
+        self.assertEqual(0, mock_store.call_count)
+
     def test_action_delete(self):
         result = ab.Action.delete(self.ctx, 'non-existent')
         self.assertIsNone(result)
@@ -337,6 +531,151 @@ class ActionBaseTest(base.SenlinTestCase):
             self.assertEqual(0, mock_call.call_count)
             mock_call.reset_mock()
 
+    @mock.patch.object(ao.Action, 'signal')
+    @mock.patch.object(dobj.Dependency, 'get_depended')
+    def test_signal_cancel(self, mock_dobj, mock_signal):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
+        action.load = mock.Mock()
+        action.set_status = mock.Mock()
+        mock_dobj.return_value = None
+
+        action.status = action.RUNNING
+        action.signal_cancel()
+
+        action.load.assert_not_called()
+        action.set_status.assert_not_called()
+        mock_dobj.assert_called_once_with(action.context, action.id)
+        mock_signal.assert_called_once_with(action.context, action.id,
+                                            action.SIG_CANCEL)
+
+    @mock.patch.object(ao.Action, 'signal')
+    @mock.patch.object(dobj.Dependency, 'get_depended')
+    def test_signal_cancel_children(self, mock_dobj, mock_signal):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
+        child_status_mock = mock.Mock()
+        children = []
+        for child_id in CHILD_IDS:
+            child = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=child_id)
+            child.status = child.READY
+            child.set_status = child_status_mock
+            children.append(child)
+        mock_dobj.return_value = CHILD_IDS
+        action.load = mock.Mock()
+        action.load.side_effect = children
+
+        action.status = action.RUNNING
+        action.signal_cancel()
+
+        mock_dobj.assert_called_once_with(action.context, action.id)
+        child_status_mock.assert_not_called()
+        self.assertEqual(3, mock_signal.call_count)
+        self.assertEqual(2, action.load.call_count)
+
+    @mock.patch.object(ao.Action, 'signal')
+    @mock.patch.object(dobj.Dependency, 'get_depended')
+    def test_signal_cancel_children_lifecycle(self, mock_dobj, mock_signal):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
+        child_status_mock = mock.Mock()
+        children = []
+        for child_id in CHILD_IDS:
+            child = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=child_id)
+            child.status = child.WAITING_LIFECYCLE_COMPLETION
+            child.set_status = child_status_mock
+            children.append(child)
+        mock_dobj.return_value = CHILD_IDS
+        action.load = mock.Mock()
+        action.load.side_effect = children
+
+        action.status = action.RUNNING
+        action.signal_cancel()
+
+        mock_dobj.assert_called_once_with(action.context, action.id)
+        self.assertEqual(2, child_status_mock.call_count)
+        self.assertEqual(3, mock_signal.call_count)
+        self.assertEqual(2, action.load.call_count)
+
+    @mock.patch.object(ao.Action, 'signal')
+    @mock.patch.object(dobj.Dependency, 'get_depended')
+    def test_signal_cancel_lifecycle(self, mock_dobj, mock_signal):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
+        action.load = mock.Mock()
+        action.set_status = mock.Mock()
+        mock_dobj.return_value = None
+
+        action.status = action.WAITING_LIFECYCLE_COMPLETION
+        action.signal_cancel()
+
+        action.load.assert_not_called()
+        action.set_status.assert_called_once_with(action.RES_CANCEL,
+                                                  'Action execution cancelled')
+        mock_dobj.assert_called_once_with(action.context, action.id)
+        mock_signal.assert_called_once_with(action.context, action.id,
+                                            action.SIG_CANCEL)
+
+    @mock.patch.object(ao.Action, 'signal')
+    @mock.patch.object(dobj.Dependency, 'get_depended')
+    def test_signal_cancel_immutable(self, mock_dobj, mock_signal):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
+        action.load = mock.Mock()
+        action.set_status = mock.Mock()
+        mock_dobj.return_value = None
+
+        action.status = action.FAILED
+        self.assertRaises(exception.ActionImmutable, action.signal_cancel)
+
+        action.load.assert_not_called()
+        action.set_status.assert_not_called()
+        mock_signal.assert_not_called()
+
+    @mock.patch.object(dobj.Dependency, 'get_depended')
+    def test_force_cancel(self, mock_dobj):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
+        action.load = mock.Mock()
+        action.set_status = mock.Mock()
+        mock_dobj.return_value = None
+
+        action.status = action.RUNNING
+        action.force_cancel()
+
+        action.load.assert_not_called()
+        action.set_status.assert_called_once_with(
+            action.RES_CANCEL, 'Action execution force cancelled')
+
+    @mock.patch.object(dobj.Dependency, 'get_depended')
+    def test_force_cancel_children(self, mock_dobj):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
+        child_status_mock = mock.Mock()
+        children = []
+        for child_id in CHILD_IDS:
+            child = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=child_id)
+            child.status = child.WAITING_LIFECYCLE_COMPLETION
+            child.set_status = child_status_mock
+            children.append(child)
+        mock_dobj.return_value = CHILD_IDS
+        action.set_status = mock.Mock()
+        action.load = mock.Mock()
+        action.load.side_effect = children
+
+        action.status = action.RUNNING
+        action.force_cancel()
+
+        mock_dobj.assert_called_once_with(action.context, action.id)
+        self.assertEqual(2, child_status_mock.call_count)
+        self.assertEqual(2, action.load.call_count)
+
+    @mock.patch.object(dobj.Dependency, 'get_depended')
+    def test_force_cancel_immutable(self, mock_dobj):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
+        action.load = mock.Mock()
+        action.set_status = mock.Mock()
+        mock_dobj.return_value = None
+
+        action.status = action.FAILED
+        self.assertRaises(exception.ActionImmutable, action.force_cancel)
+
+        action.load.assert_not_called()
+        action.set_status.assert_not_called()
+
     def test_execute_default(self):
         action = ab.Action.__new__(DummyAction, OBJID, 'BOOM', self.ctx)
         self.assertRaises(NotImplementedError,
@@ -386,12 +725,6 @@ class ActionBaseTest(base.SenlinTestCase):
         mark_cancel.assert_called_once_with(action.context, 'FAKE_ID',
                                             mock.ANY)
 
-        mark_fail.reset_mock()
-        action.set_status(action.RES_LIFECYCLE_COMPLETE, 'LIFECYCLE COMPLETE')
-        self.assertEqual(action.SUCCEEDED, action.status)
-        self.assertEqual('LIFECYCLE COMPLETE', action.status_reason)
-        mark_ready.assert_called_once_with(action.context, 'FAKE_ID', mock.ANY)
-
         mark_fail.reset_mock()
         action.set_status(action.RES_RETRY, 'BUSY')
         self.assertEqual(action.READY, action.status)
@@ -682,12 +1015,12 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
             filters={'enabled': True})
         mock_load.assert_called_once_with(action.context, policy.id)
         # last_op was updated anyway
-        self.assertIsNotNone(pb.last_op)
+        self.assertEqual(action.inputs['last_op'], pb.last_op)
         # neither pre_op nor post_op was called, because target not match
         self.assertEqual(0, mock_pre_op.call_count)
         self.assertEqual(0, mock_post_op.call_count)
 
-    def test__check_result_true(self):
+    def test_check_result_true(self):
         cluster_id = CLUSTER_ID
         action = ab.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
         action.data['status'] = policy_mod.CHECK_OK
@@ -697,7 +1030,7 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
 
         self.assertTrue(res)
 
-    def test__check_result_false(self):
+    def test_check_result_false(self):
         cluster_id = CLUSTER_ID
         action = ab.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
         action.data['status'] = policy_mod.CHECK_ERROR
@@ -768,45 +1101,11 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
             filters={'enabled': True})
         mock_load.assert_called_once_with(action.context, policy.id)
         # last_op was updated for POST check
-        self.assertIsNotNone(pb.last_op)
+        self.assertEqual(action.inputs['last_op'], pb.last_op)
         # pre_op is called, but post_op was not called
         self.assertEqual(0, policy.pre_op.call_count)
         policy.post_op.assert_called_once_with(cluster_id, action)
 
-    @mock.patch.object(cpo.ClusterPolicy, 'cooldown_inprogress')
-    @mock.patch.object(cpo.ClusterPolicy, 'get_all')
-    @mock.patch.object(policy_mod.Policy, 'load')
-    def test_policy_check_cooldown_inprogress(self, mock_load, mock_load_all,
-                                              mock_inprogress):
-        cluster_id = CLUSTER_ID
-        # Note: policy is mocked
-        policy_id = uuidutils.generate_uuid()
-        policy = mock.Mock(id=policy_id, TARGET=[('AFTER', 'OBJECT_ACTION')])
-        # Note: policy binding is created but not stored
-        pb = self._create_cp_binding(cluster_id, policy.id)
-        mock_inprogress.return_value = True
-        mock_load_all.return_value = [pb]
-        mock_load.return_value = policy
-        action = ab.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
-
-        # Do it
-        res = action.policy_check(CLUSTER_ID, 'AFTER')
-
-        self.assertIsNone(res)
-        self.assertEqual(policy_mod.CHECK_ERROR, action.data['status'])
-        self.assertEqual(
-            'Policy %s cooldown is still in progress.' % policy_id,
-            six.text_type(action.data['reason']))
-        mock_load_all.assert_called_once_with(
-            action.context, cluster_id, sort='priority',
-            filters={'enabled': True})
-        mock_load.assert_called_once_with(action.context, policy.id)
-        # last_op was updated for POST check
-        self.assertIsNotNone(pb.last_op)
-        # neither pre_op nor post_op was not called, due to cooldown
-        self.assertEqual(0, policy.pre_op.call_count)
-        self.assertEqual(0, policy.post_op.call_count)
-
     @mock.patch.object(cpo.ClusterPolicy, 'get_all')
     @mock.patch.object(policy_mod.Policy, 'load')
     @mock.patch.object(ab.Action, '_check_result')
@@ -858,6 +1157,8 @@ class ActionProcTest(base.SenlinTestCase):
     def test_action_proc_successful(self, mock_mark, mock_load,
                                     mock_event_info):
         action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
+        action.is_cancelled = mock.Mock()
+        action.is_cancelled.return_value = False
         mock_obj = mock.Mock()
         action.entity = mock_obj
         self.patchobject(action, 'execute',
@@ -878,6 +1179,8 @@ class ActionProcTest(base.SenlinTestCase):
     @mock.patch.object(ao.Action, 'mark_failed')
     def test_action_proc_failed_error(self, mock_mark, mock_load, mock_info):
         action = ab.Action(OBJID, 'CLUSTER_ACTION', self.ctx, id=ACTION_ID)
+        action.is_cancelled = mock.Mock()
+        action.is_cancelled.return_value = False
         action.entity = mock.Mock(id=CLUSTER_ID, name='fake-cluster')
 
         self.patchobject(action, 'execute', side_effect=Exception('Boom!'))
@@ -891,3 +1194,26 @@ class ActionProcTest(base.SenlinTestCase):
                                           project_safe=False)
         mock_info.assert_called_once_with(action, 'start', 'ACTION')
         mock_status.assert_called_once_with(action.RES_ERROR, 'Boom!')
+
+    @mock.patch.object(EVENT, 'info')
+    @mock.patch.object(ab.Action, 'load')
+    @mock.patch.object(ao.Action, 'mark_failed')
+    def test_action_proc_is_cancelled(self, mock_mark, mock_load, mock_info):
+        action = ab.Action(OBJID, 'CLUSTER_ACTION', self.ctx, id=ACTION_ID)
+        action.is_cancelled = mock.Mock()
+        action.is_cancelled.return_value = True
+        action.entity = mock.Mock(id=CLUSTER_ID, name='fake-cluster')
+
+        mock_status = self.patchobject(action, 'set_status')
+        mock_load.return_value = action
+
+        res = ab.ActionProc(self.ctx, 'ACTION')
+        self.assertIs(True, res)
+
+        mock_load.assert_called_once_with(self.ctx, action_id='ACTION',
+                                          project_safe=False)
+
+        mock_info.assert_not_called()
+        mock_status.assert_called_once_with(
+            action.RES_CANCEL,
+            'CLUSTER_ACTION [%s] cancelled' % ACTION_ID[:8])
