@@ -1137,7 +1137,7 @@ class ClusterTest(base.SenlinTestCase):
                                self.ctx, req.obj_to_primitive())
 
         self.assertEqual(exc.BadRequest, ex.exc_info[0])
-        self.assertEqual("The 'number' must be positive integer for "
+        self.assertEqual("The 'number' must be non-negative integer for "
                          "adjustment type 'EXACT_CAPACITY'.",
                          six.text_type(ex.exc_info[1]))
 
@@ -1512,6 +1512,74 @@ class ClusterTest(base.SenlinTestCase):
         )
         notify.assert_called_once_with()
 
+    @mock.patch.object(am.Action, 'create')
+    @mock.patch.object(co.Cluster, 'find')
+    @mock.patch.object(dispatcher, 'start_action')
+    def test_cluster_recover_rebuild(self, notify, mock_find, mock_action):
+        x_cluster = mock.Mock(id='CID')
+        mock_find.return_value = x_cluster
+        mock_action.return_value = 'ACTION_ID'
+        req = orco.ClusterRecoverRequest(identity='C1',
+                                         params={'operation': 'REBUILD'})
+
+        result = self.eng.cluster_recover(self.ctx, req.obj_to_primitive())
+
+        self.assertEqual({'action': 'ACTION_ID'}, result)
+        mock_find.assert_called_once_with(self.ctx, 'C1')
+        mock_action.assert_called_once_with(
+            self.ctx, 'CID', consts.CLUSTER_RECOVER,
+            name='cluster_recover_CID',
+            cause=consts.CAUSE_RPC,
+            status=am.Action.READY,
+            inputs={'operation': 'REBUILD'},
+        )
+        notify.assert_called_once_with()
+
+    @mock.patch.object(am.Action, 'create')
+    @mock.patch.object(co.Cluster, 'find')
+    @mock.patch.object(dispatcher, 'start_action')
+    def test_cluster_recover_reboot(self, notify, mock_find, mock_action):
+        x_cluster = mock.Mock(id='CID')
+        mock_find.return_value = x_cluster
+        mock_action.return_value = 'ACTION_ID'
+        req = orco.ClusterRecoverRequest(identity='C1',
+                                         params={'operation': 'REBOOT'})
+
+        result = self.eng.cluster_recover(self.ctx, req.obj_to_primitive())
+
+        self.assertEqual({'action': 'ACTION_ID'}, result)
+        mock_find.assert_called_once_with(self.ctx, 'C1')
+        mock_action.assert_called_once_with(
+            self.ctx, 'CID', consts.CLUSTER_RECOVER,
+            name='cluster_recover_CID',
+            cause=consts.CAUSE_RPC,
+            status=am.Action.READY,
+            inputs={'operation': 'REBOOT'},
+        )
+        notify.assert_called_once_with()
+
+    @mock.patch.object(am.Action, 'create')
+    @mock.patch.object(co.Cluster, 'find')
+    @mock.patch.object(dispatcher, 'start_action')
+    def test_cluster_recover_default(self, notify, mock_find, mock_action):
+        x_cluster = mock.Mock(id='CID')
+        mock_find.return_value = x_cluster
+        mock_action.return_value = 'ACTION_ID'
+        req = orco.ClusterRecoverRequest(identity='C1')
+
+        result = self.eng.cluster_recover(self.ctx, req.obj_to_primitive())
+
+        self.assertEqual({'action': 'ACTION_ID'}, result)
+        mock_find.assert_called_once_with(self.ctx, 'C1')
+        mock_action.assert_called_once_with(
+            self.ctx, 'CID', consts.CLUSTER_RECOVER,
+            name='cluster_recover_CID',
+            cause=consts.CAUSE_RPC,
+            status=am.Action.READY,
+            inputs={}
+        )
+        notify.assert_called_once_with()
+
     @mock.patch.object(co.Cluster, 'find')
     def test_cluster_recover_cluster_not_found(self, mock_find):
         mock_find.side_effect = exc.ResourceNotFound(type='cluster',
@@ -1544,6 +1612,46 @@ class ClusterTest(base.SenlinTestCase):
                          six.text_type(ex.exc_info[1]))
         mock_find.assert_called_once_with(self.ctx, 'Bogus')
 
+    @mock.patch.object(co.Cluster, 'find')
+    def test_cluster_recover_invalid_operation(self, mock_find):
+        x_cluster = mock.Mock(id='CID')
+        mock_find.return_value = x_cluster
+
+        req = orco.ClusterRecoverRequest(identity='Bogus',
+                                         params={'operation': 'fake'})
+
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.cluster_recover,
+                               self.ctx, req.obj_to_primitive())
+
+        self.assertEqual(exc.BadRequest, ex.exc_info[0])
+        self.assertEqual("Operation value 'fake' has to be one of the "
+                         "following: REBOOT, REBUILD, RECREATE.",
+                         six.text_type(ex.exc_info[1]))
+        mock_find.assert_called_once_with(self.ctx, 'Bogus')
+
+    @mock.patch.object(co.Cluster, 'find')
+    def test_cluster_recover_invalid_operation_params(self, mock_find):
+        x_cluster = mock.Mock(id='CID')
+        mock_find.return_value = x_cluster
+
+        req = orco.ClusterRecoverRequest(
+            identity='Bogus',
+            params={'operation': 'reboot',
+                    'operation_params': {'type': 'blah'}
+                    }
+        )
+
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.cluster_recover,
+                               self.ctx, req.obj_to_primitive())
+
+        self.assertEqual(exc.BadRequest, ex.exc_info[0])
+        self.assertEqual("Type field 'blah' in operation_params has to be one "
+                         "of the following: SOFT, HARD.",
+                         six.text_type(ex.exc_info[1]))
+        mock_find.assert_called_once_with(self.ctx, 'Bogus')
+
     @mock.patch.object(am.Action, 'create')
     @mock.patch.object(co.Cluster, 'find')
     @mock.patch.object(dispatcher, 'start_action')
@@ -1570,7 +1678,7 @@ class ClusterTest(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'find')
     @mock.patch.object(po.Profile, 'get')
-    def test__validate_replace_nodes(self, mock_profile, mock_node):
+    def test_validate_replace_nodes(self, mock_profile, mock_node):
         cluster = mock.Mock(id='CID', profile_id='FAKE_ID')
         mock_profile.return_value = mock.Mock(type='FAKE_TYPE')
         mock_node.side_effect = [
@@ -1595,8 +1703,8 @@ class ClusterTest(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'find')
     @mock.patch.object(po.Profile, 'get')
-    def test__validate_replace_nodes_old_missing(self, mock_profile,
-                                                 mock_node):
+    def test_validate_replace_nodes_old_missing(self, mock_profile,
+                                                mock_node):
         c = mock.Mock(id='CID', profile_id='FAKE_ID')
         mock_node.side_effect = exc.ResourceNotFound(type='node', id='OLD')
 
@@ -1610,8 +1718,8 @@ class ClusterTest(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'find')
     @mock.patch.object(po.Profile, 'get')
-    def test__validate_replace_nodes_new_missing(self, mock_profile,
-                                                 mock_node):
+    def test_validate_replace_nodes_new_missing(self, mock_profile,
+                                                mock_node):
         c = mock.Mock(id='CID', profile_id='FAKE_ID')
         mock_node.side_effect = [
             mock.Mock(),
@@ -1632,8 +1740,8 @@ class ClusterTest(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'find')
     @mock.patch.object(po.Profile, 'get')
-    def test__validate_replace_nodes_old_not_member(self, mock_profile,
-                                                    mock_node):
+    def test_validate_replace_nodes_old_not_member(self, mock_profile,
+                                                   mock_node):
         c = mock.Mock(id='CID', profile_id='FAKE_ID')
         mock_node.side_effect = [
             mock.Mock(cluster_id='OTHER'),
@@ -1654,8 +1762,8 @@ class ClusterTest(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'find')
     @mock.patch.object(po.Profile, 'get')
-    def test__validate_replace_nodes_new_not_orphan(self, mock_profile,
-                                                    mock_node):
+    def test_validate_replace_nodes_new_not_orphan(self, mock_profile,
+                                                   mock_node):
         c = mock.Mock(id='CID', profile_id='FAKE_ID')
         mock_node.side_effect = [
             mock.Mock(cluster_id='CID'),
@@ -1676,8 +1784,8 @@ class ClusterTest(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'find')
     @mock.patch.object(po.Profile, 'get')
-    def test__validate_replace_nodes_new_bad_status(self, mock_profile,
-                                                    mock_node):
+    def test_validate_replace_nodes_new_bad_status(self, mock_profile,
+                                                   mock_node):
         c = mock.Mock(id='CID', profile_id='FAKE_ID')
         mock_node.side_effect = [
             mock.Mock(cluster_id='CID'),
@@ -1697,8 +1805,8 @@ class ClusterTest(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'find')
     @mock.patch.object(po.Profile, 'get')
-    def test__validate_replace_nodes_mult_err(self, mock_profile,
-                                              mock_node):
+    def test_validate_replace_nodes_mult_err(self, mock_profile,
+                                             mock_node):
         c = mock.Mock(id='CID', profile_id='FAKE_ID')
         mock_node.side_effect = [
             mock.Mock(id='OLD1', cluster_id='CID'),
@@ -1721,7 +1829,7 @@ class ClusterTest(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'find')
     @mock.patch.object(po.Profile, 'get')
-    def test__validate_replace_nodes_new_profile_type_mismatch(
+    def test_validate_replace_nodes_new_profile_type_mismatch(
             self, mock_profile, mock_node):
         c = mock.Mock(id='CID', profile_id='FAKE_CLUSTER_PROFILE')
         mock_profile.side_effect = [
