@@ -251,11 +251,15 @@ class TestNode(base.SenlinTestCase):
                                     'Creation succeeded',
                                     physical_id=physical_id)
 
-    def test_node_create_not_init(self):
+    @mock.patch.object(nodem.Node, 'set_status')
+    def test_node_create_not_init(self, mock_status):
         node = nodem.Node('node1', PROFILE_ID, CLUSTER_ID, self.context)
         node.status = 'NOT_INIT'
-        res = node.do_create(self.context)
+        res, reason = node.do_create(self.context)
         self.assertFalse(res)
+        self.assertEqual('Node must be in INIT status', reason)
+        mock_status.assert_any_call(self.context, consts.NS_ERROR,
+                                    'Node must be in INIT status')
 
     @mock.patch.object(nodem.Node, 'set_status')
     @mock.patch.object(pb.Profile, 'create_object')
@@ -265,9 +269,10 @@ class TestNode(base.SenlinTestCase):
         mock_create.side_effect = exception.EResourceCreation(
             type='PROFILE', message='Boom', resource_id='test_id')
 
-        res = node.do_create(self.context)
+        res, reason = node.do_create(self.context)
 
         self.assertFalse(res)
+        self.assertEqual(str(reason), 'Failed in creating PROFILE: Boom.')
         mock_status.assert_any_call(self.context, consts.NS_CREATING,
                                     'Creation in progress')
         mock_status.assert_any_call(self.context, consts.NS_ERROR,
@@ -452,9 +457,8 @@ class TestNode(base.SenlinTestCase):
         self.assertEqual(mock_migrate.return_value.index, node.index)
         self.assertIsNotNone(node.updated_at)
 
-    @mock.patch.object(node_obj.Node, 'migrate')
     @mock.patch.object(pb.Profile, 'join_cluster')
-    def test_node_join_fail_profile_call(self, mock_join, mock_migrate):
+    def test_node_join_fail_profile_call(self, mock_join):
         node = nodem.Node('node1', PROFILE_ID, None, self.context)
         node.id = uuidutils.generate_uuid()
         mock_join.return_value = False
@@ -464,11 +468,6 @@ class TestNode(base.SenlinTestCase):
 
         self.assertFalse(res)
         mock_join.assert_called_once_with(self.context, node, cluster_id)
-        mock_migrate.assert_has_calls([
-            mock.call(self.context, node.id, cluster_id, mock.ANY),
-            mock.call(self.context, node.id, None, mock.ANY),
-        ])
-        self.assertEqual('', node.cluster_id)
         self.assertEqual(-1, node.index)
 
     @mock.patch.object(node_obj.Node, 'migrate')
@@ -595,6 +594,24 @@ class TestNode(base.SenlinTestCase):
             % node.physical_id,
             physical_id=None)
 
+    @mock.patch.object(pb.Profile, 'healthcheck_object')
+    def test_node_healthcheck(self, mock_healthcheck):
+        node = nodem.Node('node1', PROFILE_ID, '')
+        node.status = consts.NS_ACTIVE
+        node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
+        mock_healthcheck.return_value = True
+        res = node.do_healthcheck(self.context)
+
+        self.assertTrue(res)
+        mock_healthcheck.assert_called_once_with(self.context, node)
+
+    def test_node_healthcheck_no_physical_id(self):
+        node = nodem.Node('node1', PROFILE_ID, '')
+
+        res = node.do_healthcheck(self.context)
+
+        self.assertFalse(res)
+
     @mock.patch.object(nodem.Node, 'set_status')
     @mock.patch.object(pb.Profile, 'recover_object')
     def test_node_recover_new_object(self, mock_recover, mock_status):
@@ -609,10 +626,10 @@ class TestNode(base.SenlinTestCase):
 
         # action = node_action.NodeAction(node.id, 'ACTION', self.ctx)
 
-        mock_recover.return_value = new_id
+        mock_recover.return_value = new_id, True
         mock_status.side_effect = set_status
         action = mock.Mock()
-        action.inputs = {'operation': ['SWIM', 'DANCE']}
+        action.inputs = {'operation': 'SWIM'}
 
         res = node.do_recover(self.context, action)
 
@@ -636,7 +653,7 @@ class TestNode(base.SenlinTestCase):
     def test_node_recover_in_place(self, mock_recover, mock_status):
         node = nodem.Node('node1', PROFILE_ID, None)
         node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
-        mock_recover.return_value = node.physical_id
+        mock_recover.return_value = node.physical_id, True
         action = mock.Mock(inputs={})
 
         res = node.do_recover(self.context, action)
@@ -673,12 +690,12 @@ class TestNode(base.SenlinTestCase):
         def set_status(*args, **kwargs):
             if args[1] == 'ACTIVE':
                 node.physical_id = new_id
-                node.data = {'recovery': 'RECREATE'}
+                node.data = {'recovery': 'recreate'}
 
         node = nodem.Node('node1', PROFILE_ID, '')
         node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
         new_id = '166db83b-b4a4-49ef-96a8-6c0fdd882d1a'
-        mock_recover.return_value = new_id
+        mock_recover.return_value = new_id, True
         mock_status.side_effect = set_status
         mock_check = self.patchobject(pb.Profile, 'check_object')
         mock_check.return_value = False
@@ -712,49 +729,12 @@ class TestNode(base.SenlinTestCase):
         node = nodem.Node('node1', PROFILE_ID, '', id='fake')
         node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
         new_id = '166db83b-b4a4-49ef-96a8-6c0fdd882d1a'
-        mock_recover.return_value = new_id
+        mock_recover.return_value = new_id, True
         mock_status.side_effect = set_status
         mock_check = self.patchobject(pb.Profile, 'check_object')
         mock_check.return_value = False
         action = mock.Mock(
-            outputs={}, inputs={'operation': [{'name': 'RECREATE'}],
-                                'check': True})
-
-        res = node.do_recover(self.context, action)
-
-        self.assertTrue(res)
-        mock_check.assert_called_once_with(self.context, node)
-        mock_recover.assert_called_once_with(
-            self.context, node, **action.inputs)
-        self.assertEqual('node1', node.name)
-        self.assertEqual(new_id, node.physical_id)
-        self.assertEqual(PROFILE_ID, node.profile_id)
-        mock_status.assert_has_calls([
-            mock.call(self.context, 'RECOVERING',
-                      reason='Recovery in progress'),
-            mock.call(self.context, consts.NS_ACTIVE,
-                      reason='Recovery succeeded',
-                      physical_id=new_id,
-                      data={'recovery': 'RECREATE'})])
-
-    @mock.patch.object(nodem.Node, 'set_status')
-    @mock.patch.object(pb.Profile, 'recover_object')
-    def test_node_recover_mult_rebuild(self, mock_recover, mock_status):
-        def set_status(*args, **kwargs):
-            if args[1] == 'ACTIVE':
-                node.physical_id = new_id
-                node.data = {'recovery': 'RECREATE'}
-
-        node = nodem.Node('node1', PROFILE_ID, '', id='fake')
-        node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
-        new_id = '166db83b-b4a4-49ef-96a8-6c0fdd882d1a'
-        mock_recover.return_value = new_id
-        mock_status.side_effect = set_status
-        mock_check = self.patchobject(pb.Profile, 'check_object')
-        mock_check.return_value = False
-        action = mock.Mock(
-            outputs={}, inputs={'operation': [{'name': 'REBOOT'},
-                                              {'name': 'REBUILD'}],
+            outputs={}, inputs={'operation': 'RECREATE',
                                 'check': True})
 
         res = node.do_recover(self.context, action)
@@ -785,7 +765,7 @@ class TestNode(base.SenlinTestCase):
         node = nodem.Node('node1', PROFILE_ID, '')
         node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
         new_id = '166db83b-b4a4-49ef-96a8-6c0fdd882d1a'
-        mock_recover.return_value = new_id
+        mock_recover.return_value = new_id, True
         mock_status.side_effect = set_status
         mock_check = self.patchobject(pb.Profile, 'check_object')
         mock_check.side_effect = exception.EResourceOperation(
@@ -794,7 +774,7 @@ class TestNode(base.SenlinTestCase):
             id=node.physical_id,
             reason='Boom!'
         )
-        action = mock.Mock(inputs={'operation': ['boom'],
+        action = mock.Mock(inputs={'operation': 'boom',
                                    'check': True})
 
         res = node.do_recover(self.context, action)
@@ -819,8 +799,8 @@ class TestNode(base.SenlinTestCase):
     def test_node_recover_failed_recover(self, mock_recover, mock_status):
         node = nodem.Node('node1', PROFILE_ID, None)
         node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
-        mock_recover.return_value = None
-        action = mock.Mock(inputs={'operation': [{'name': 'RECREATE'}]})
+        mock_recover.return_value = node.physical_id, None
+        action = mock.Mock(inputs={'operation': 'RECREATE'})
 
         res = node.do_recover(self.context, action)
 
@@ -831,20 +811,110 @@ class TestNode(base.SenlinTestCase):
             mock.call(self.context, consts.NS_ERROR,
                       reason='Recovery failed')])
 
-    def test_node_recover_no_physical_id(self):
+    def test_node_recover_no_physical_id_reboot_op(self):
         node = nodem.Node('node1', PROFILE_ID, None)
-        action = mock.Mock()
+        action = mock.Mock(inputs={'operation': 'REBOOT'})
+
+        res = node.do_recover(self.context, action)
+
+        self.assertFalse(res)
+
+    def test_node_recover_no_physical_id_rebuild_op(self):
+        node = nodem.Node('node1', PROFILE_ID, None)
+        action = mock.Mock(inputs={'operation': 'REBUILD'})
 
         res = node.do_recover(self.context, action)
 
         self.assertFalse(res)
 
+    @mock.patch.object(nodem.Node, 'set_status')
+    @mock.patch.object(pb.Profile, 'recover_object')
+    def test_node_recover_no_physical_id_no_op(self, mock_recover,
+                                               mock_status):
+        def set_status(*args, **kwargs):
+            if args[1] == 'ACTIVE':
+                node.physical_id = new_id
+                node.data = {'recovery': 'RECREATE'}
+
+        node = nodem.Node('node1', PROFILE_ID, '', id='fake')
+        new_id = '166db83b-b4a4-49ef-96a8-6c0fdd882d1a'
+        mock_recover.return_value = new_id, True
+        mock_status.side_effect = set_status
+        mock_check = self.patchobject(pb.Profile, 'check_object')
+        mock_check.return_value = False
+        action = mock.Mock(
+            outputs={}, inputs={})
+
+        res = node.do_recover(self.context, action)
+
+        self.assertTrue(res)
+        mock_check.assert_not_called()
+        mock_recover.assert_called_once_with(
+            self.context, node, **action.inputs)
+        self.assertEqual('node1', node.name)
+        self.assertEqual(new_id, node.physical_id)
+        self.assertEqual(PROFILE_ID, node.profile_id)
+        mock_status.assert_has_calls([
+            mock.call(self.context, 'RECOVERING',
+                      reason='Recovery in progress'),
+            mock.call(self.context, consts.NS_ACTIVE,
+                      reason='Recovery succeeded',
+                      physical_id=new_id,
+                      data={'recovery': 'RECREATE'})])
+
+    @mock.patch.object(nodem.Node, 'set_status')
+    @mock.patch.object(pb.Profile, 'recover_object')
+    def test_node_recover_no_physical_id_recreate_op(self, mock_recover,
+                                                     mock_status):
+        def set_status(*args, **kwargs):
+            if args[1] == 'ACTIVE':
+                node.physical_id = new_id
+                node.data = {'recovery': 'RECREATE'}
+
+        node = nodem.Node('node1', PROFILE_ID, '', id='fake')
+        new_id = '166db83b-b4a4-49ef-96a8-6c0fdd882d1a'
+        mock_recover.return_value = new_id, True
+        mock_status.side_effect = set_status
+        mock_check = self.patchobject(pb.Profile, 'check_object')
+        mock_check.return_value = False
+        action = mock.Mock(
+            outputs={}, inputs={'operation': 'RECREATE',
+                                'check': True})
+
+        res = node.do_recover(self.context, action)
+
+        self.assertTrue(res)
+        mock_check.assert_called_once_with(self.context, node)
+        mock_recover.assert_called_once_with(
+            self.context, node, **action.inputs)
+        self.assertEqual('node1', node.name)
+        self.assertEqual(new_id, node.physical_id)
+        self.assertEqual(PROFILE_ID, node.profile_id)
+        mock_status.assert_has_calls([
+            mock.call(self.context, 'RECOVERING',
+                      reason='Recovery in progress'),
+            mock.call(self.context, consts.NS_ACTIVE,
+                      reason='Recovery succeeded',
+                      physical_id=new_id,
+                      data={'recovery': 'RECREATE'})])
+
     @mock.patch.object(nodem.Node, 'set_status')
     def test_node_recover_operation_not_support(self, mock_set_status):
         node = nodem.Node('node1', PROFILE_ID, None)
         node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
         action = mock.Mock(
-            outputs={}, inputs={'operation': [{'name': 'foo'}]})
+            outputs={}, inputs={'operation': 'foo'})
+
+        res = node.do_recover(self.context, action)
+        self.assertEqual({}, action.outputs)
+        self.assertFalse(res)
+
+    @mock.patch.object(nodem.Node, 'set_status')
+    def test_node_recover_operation_not_string(self, mock_set_status):
+        node = nodem.Node('node1', PROFILE_ID, None)
+        node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
+        action = mock.Mock(
+            outputs={}, inputs={'operation': 'foo'})
 
         res = node.do_recover(self.context, action)
         self.assertEqual({}, action.outputs)
@@ -900,6 +970,28 @@ class TestNode(base.SenlinTestCase):
         ])
         x_profile.handle_dance.assert_called_once_with(node, style='tango')
 
+    def _verify_execution_create_args(self, expected_name,
+                                      expected_inputs_dict, wfc):
+        wfc.execution_create.assert_called_once_with(mock.ANY, mock.ANY)
+        actual_call_args, call_kwargs = wfc.execution_create.call_args
+
+        # execution_create parameters are name and inputs
+        actual_call_name, actual_call_inputs = actual_call_args
+
+        # actual_call_inputs is string representation of a dictionary.
+        # convert actual_call_inputs to json, then dump it back as string
+        # sorted by key
+        final_actual_call_inputs = jsonutils.dumps(
+            jsonutils.loads(actual_call_inputs), sort_keys=True)
+
+        # dump expected_inputs_dict as string sorted by key
+        final_expected_inputs = jsonutils.dumps(
+            expected_inputs_dict, sort_keys=True)
+
+        # compare the sorted input strings along with the names
+        self.assertEqual(actual_call_name, expected_name)
+        self.assertEqual(final_actual_call_inputs, final_expected_inputs)
+
     def test_run_workflow(self):
         node = nodem.Node('node1', PROFILE_ID, 'FAKE_CLUSTER')
         node.physical_id = 'FAKE_NODE'
@@ -932,13 +1024,12 @@ class TestNode(base.SenlinTestCase):
             {'bar': 'baz'}, scope='private')
 
         final_dict = {
-            'FAKE_KEY1': 'FAKE_VALUE1',
-            'FAKE_KEY2': 'FAKE_VALUE2',
             'cluster_id': 'FAKE_CLUSTER',
             'node_id': 'FAKE_NODE',
+            'FAKE_KEY1': 'FAKE_VALUE1',
+            'FAKE_KEY2': 'FAKE_VALUE2',
         }
-        final_inputs = jsonutils.dumps(final_dict)
-        wfc.execution_create.assert_called_once_with('foo', final_inputs)
+        self._verify_execution_create_args('foo', final_dict, wfc)
 
     def test_run_workflow_no_physical_id(self):
         node = nodem.Node('node1', PROFILE_ID, 'FAKE_CLUSTER')
@@ -975,13 +1066,12 @@ class TestNode(base.SenlinTestCase):
         wfc.workflow_find.assert_called_once_with('foo')
         self.assertEqual(0, wfc.workflow_create.call_count)
         final_dict = {
-            'FAKE_KEY1': 'FAKE_VALUE1',
-            'FAKE_KEY2': 'FAKE_VALUE2',
             'cluster_id': 'FAKE_CLUSTER',
             'node_id': 'FAKE_NODE',
+            'FAKE_KEY1': 'FAKE_VALUE1',
+            'FAKE_KEY2': 'FAKE_VALUE2',
         }
-        final_inputs = jsonutils.dumps(final_dict)
-        wfc.execution_create.assert_called_once_with('foo', final_inputs)
+        self._verify_execution_create_args('foo', final_dict, wfc)
 
     def test_run_workflow_failed_creation(self):
         node = nodem.Node('node1', PROFILE_ID, 'FAKE_CLUSTER')
@@ -1050,10 +1140,9 @@ class TestNode(base.SenlinTestCase):
         wfc.workflow_create.assert_called_once_with(
             {'bar': 'baz'}, scope='private')
         final_dict = {
-            'FAKE_KEY1': 'FAKE_VALUE1',
-            'FAKE_KEY2': 'FAKE_VALUE2',
             'cluster_id': 'FAKE_CLUSTER',
             'node_id': 'FAKE_NODE',
+            'FAKE_KEY1': 'FAKE_VALUE1',
+            'FAKE_KEY2': 'FAKE_VALUE2',
         }
-        final_inputs = jsonutils.dumps(final_dict)
-        wfc.execution_create.assert_called_once_with('foo', final_inputs)
+        self._verify_execution_create_args('foo', final_dict, wfc)
