@@ -697,8 +697,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'get')
     @mock.patch.object(no.Node, 'update')
-    def test__add_member(self, m_node_update, m_node_get,
-                         m_extract, m_load):
+    def test_add_member(self, m_node_update, m_node_get,
+                        m_extract, m_load):
         node1 = mock.Mock(id='NODE1_ID', data={})
         node2 = mock.Mock(id='NODE2_ID', data={})
         action = mock.Mock(context='action_context',
@@ -754,8 +754,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'get')
     @mock.patch.object(no.Node, 'update')
-    def test__add_member_fail(self, m_node_update, m_node_get,
-                              m_extract, m_load):
+    def test_add_member_fail(self, m_node_update, m_node_get,
+                             m_extract, m_load):
         node1 = mock.Mock(id='NODE1_ID', data={})
         action = mock.Mock(context='action_context',
                            action=consts.CLUSTER_RESIZE,
@@ -808,7 +808,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         cid = 'CLUSTER_ID'
         cluster = mock.Mock(user='user1', project='project1')
         action = mock.Mock(data={}, context=ctx, action=consts.NODE_CREATE,
-                           node=mock.Mock(id='NODE_ID'))
+                           node=mock.Mock(id='NODE_ID'),
+                           inputs={'action_result': 'OK'})
         action.entity = cluster
         cp = mock.Mock()
         m_load.return_value = cp
@@ -840,7 +841,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
                                'creation': {
                                    'nodes': ['NODE1_ID', 'NODE2_ID']
                                }
-                           })
+                           },
+                           inputs={'action_result': 'OK'})
         action.entity = cluster
         candidates = ['NODE1_ID', 'NODE2_ID']
         m_get.return_value = candidates
@@ -870,7 +872,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         action = mock.Mock(context='action_context',
                            action=consts.NODE_RECOVER,
                            data={},
-                           outputs={})
+                           outputs={},
+                           inputs={'action_result': 'OK'})
         action.entity = node
         m_recovery.return_value = ['NODE1']
         m_get.return_value = ['NODE1']
@@ -899,7 +902,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         cluster_id = 'CLUSTER_ID'
         action = mock.Mock(data={'creation': {'nodes': ['NODE1_ID']}},
                            context='action_context',
-                           action=consts.CLUSTER_RESIZE)
+                           action=consts.CLUSTER_RESIZE,
+                           inputs={'action_result': 'OK'})
 
         cp = mock.Mock()
         m_load.return_value = cp
@@ -921,8 +925,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'get')
     @mock.patch.object(no.Node, 'update')
-    def test__remove_member(self, m_node_update, m_node_get,
-                            m_extract, m_load):
+    def test_remove_member(self, m_node_update, m_node_get,
+                           m_extract, m_load):
         node1 = mock.Mock(id='NODE1', data={'lb_member': 'MEM_ID1'})
         node2 = mock.Mock(id='NODE2', data={'lb_member': 'MEM_ID2'})
         action = mock.Mock(
@@ -976,8 +980,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'get')
     @mock.patch.object(no.Node, 'update')
-    def test__remove_member_not_in_pool(self, m_node_update, m_node_get,
-                                        m_extract, m_load):
+    def test_remove_member_not_in_pool(self, m_node_update, m_node_get,
+                                       m_extract, m_load):
         node1 = mock.Mock(id='NODE1', data={'lb_member': 'MEM_ID1'})
         node2 = mock.Mock(id='NODE2', data={})
         action = mock.Mock(
@@ -1025,8 +1029,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'get')
     @mock.patch.object(no.Node, 'update')
-    def test__remove_member_fail(self, m_node_update, m_node_get,
-                                 m_extract, m_load):
+    def test_remove_member_fail(self, m_node_update, m_node_get,
+                                m_extract, m_load):
         node1 = mock.Mock(id='NODE1', data={'lb_member': 'MEM_ID1'})
         action = mock.Mock(
             context='action_context', action=consts.CLUSTER_DEL_NODES,
@@ -1120,7 +1124,7 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
                                          mock.ANY, self.lb_driver)
 
     @mock.patch.object(no.Node, 'update')
-    def test__process_recovery_not_lb_member(self, m_update, m1, m2):
+    def test_process_recovery_not_lb_member(self, m_update, m1, m2):
         node = mock.Mock(id='NODE', data={})
         action = mock.Mock(
             action=consts.NODE_RECOVER,
@@ -1133,11 +1137,11 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         res = policy._process_recovery(['NODE'], cp, self.lb_driver, action)
 
         self.assertEqual(['NODE'], res)
-        m_update.assert_called_once_with(action.context, {'data': {}})
+        m_update.assert_called_once_with(action.context, 'NODE', {'data': {}})
 
     @mock.patch.object(no.Node, 'update')
     @mock.patch.object(lb_policy.LoadBalancingPolicy, '_remove_member')
-    def test__process_recovery_reboot(self, m_remove, m_update, m1, m2):
+    def test_process_recovery_reboot(self, m_remove, m_update, m1, m2):
         node = mock.Mock(id='NODE', data={'lb_member': 'mem_1'})
         action = mock.Mock(
             action=consts.NODE_RECOVER,
@@ -1156,7 +1160,7 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'update')
     @mock.patch.object(lb_policy.LoadBalancingPolicy, '_remove_member')
-    def test__process_recovery_recreate(self, m_remove, m_update, m1, m2):
+    def test_process_recovery_recreate(self, m_remove, m_update, m1, m2):
         node = mock.Mock(id='NODE', data={'lb_member': 'mem_1',
                                           'recovery': 'RECREATE'})
         action = mock.Mock(
@@ -1172,4 +1176,4 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         self.assertEqual(['NODE'], res)
         m_remove.assert_called_once_with(action.context, ['NODE'], cp,
                                          self.lb_driver, handle_err=False)
-        m_update.assert_called_once_with(action.context, {'data': {}})
+        m_update.assert_called_once_with(action.context, 'NODE', {'data': {}})
