@@ -178,7 +178,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         self.assertEqual(msg, ex.message)
 
     @mock.patch.object(node.Node, 'load')
-    def test__get_host_node_found_by_node(self, mock_load):
+    def test_get_host_node_found_by_node(self, mock_load):
         node = mock.Mock()
         mock_load.return_value = node
         ctx = mock.Mock()
@@ -190,7 +190,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         mock_load.assert_called_once_with(ctx, node_id='host_node')
 
     @mock.patch.object(dp.DockerProfile, '_get_random_node')
-    def test__get_host_node_found_by_cluster(self, mock_get):
+    def test_get_host_node_found_by_cluster(self, mock_get):
         node = mock.Mock()
         mock_get.return_value = node
         ctx = mock.Mock()
@@ -202,7 +202,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         mock_get.assert_called_once_with(ctx, 'host_cluster')
 
     @mock.patch.object(node.Node, 'load')
-    def test__get_host_node_not_found(self, mock_load):
+    def test_get_host_node_not_found(self, mock_load):
         mock_load.side_effect = exc.ResourceNotFound(type='node',
                                                      id='fake_node')
         profile = dp.DockerProfile('container', self.spec)
@@ -218,7 +218,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
     @mock.patch.object(node.Node, 'load')
     @mock.patch.object(no.Node, 'get_all_by_cluster')
     @mock.patch.object(cluster.Cluster, 'load')
-    def test__get_random_node(self, mock_cluster, mock_nodes, mock_load):
+    def test_get_random_node(self, mock_cluster, mock_nodes, mock_load):
         cluster = mock.Mock()
         mock_cluster.return_value = cluster
         node1 = mock.Mock()
@@ -240,7 +240,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         self.assertIn(n, [node1, node2])
 
     @mock.patch.object(cluster.Cluster, 'load')
-    def test__get_random_node_cluster_not_found(self, mock_load):
+    def test_get_random_node_cluster_not_found(self, mock_load):
         mock_load.side_effect = exc.ResourceNotFound(type='cluster',
                                                      id='host_cluster')
         ctx = mock.Mock()
@@ -255,7 +255,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
 
     @mock.patch.object(no.Node, 'get_all_by_cluster')
     @mock.patch.object(cluster.Cluster, 'load')
-    def test__get_random_node_empty_cluster(self, mock_cluster, mock_nodes):
+    def test_get_random_node_empty_cluster(self, mock_cluster, mock_nodes):
         cluster = mock.Mock()
         mock_cluster.return_value = cluster
         mock_nodes.return_value = []
@@ -476,6 +476,119 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         self.assertRaises(exc.EResourceDeletion,
                           profile.do_delete, obj)
 
+    @mock.patch.object(dp.DockerProfile, 'docker')
+    def test_update_name(self, mock_docker):
+        x_docker = mock.Mock()
+        x_docker = mock_docker.return_value
+        obj = mock.Mock(physical_id='FAKE_ID')
+
+        docker = dp.DockerProfile('container', self.spec)
+        res = docker._update_name(obj, 'NEW_NAME')
+
+        self.assertIsNone(res)
+        x_docker.rename.assert_called_once_with('FAKE_ID', 'NEW_NAME')
+
+    @mock.patch.object(dp.DockerProfile, 'docker')
+    def test_update_name_docker_failure(self, mock_docker):
+        x_docker = mock.Mock()
+        x_docker = mock_docker.return_value
+        x_docker.rename.side_effect = exc.InternalError(message='BOOM')
+        obj = mock.Mock(physical_id='FAKE_ID')
+        docker = dp.DockerProfile('container', self.spec)
+
+        ex = self.assertRaises(exc.EResourceUpdate,
+                               docker._update_name,
+                               obj, 'NEW_NAME')
+
+        self.assertEqual("Failed in updating container 'FAKE_ID': BOOM.",
+                         six.text_type(ex))
+        x_docker.rename.assert_called_once_with('FAKE_ID', 'NEW_NAME')
+
+    @mock.patch.object(dp.DockerProfile, 'docker')
+    def test_do_update(self, mock_docker):
+        obj = mock.Mock(physical_id='FAKE_ID')
+        docker = dp.DockerProfile('container', self.spec)
+
+        new_spec = {
+            'type': 'container.dockerinc.docker',
+            'version': '1.0',
+            'properties': {
+                'context': {
+                    'region_name': 'RegionOne'
+                },
+                'name': 'new_name',
+                'image': 'hello-world',
+                'command': '/bin/sleep 30',
+                'port': 2375,
+                'host_node': 'fake_node',
+            }
+        }
+        new_profile = dp.DockerProfile('u', new_spec)
+        res = docker.do_update(obj, new_profile)
+
+        self.assertTrue(res)
+
+    @mock.patch.object(dp.DockerProfile, 'docker')
+    def test_do_update_no_new_profile(self, mock_docker):
+        obj = mock.Mock(physical_id='FAKE_ID')
+        docker = dp.DockerProfile('container', self.spec)
+
+        params = {}
+        res = docker.do_update(obj, params)
+
+        self.assertFalse(res)
+
+    def test_do_update_no_physical_id(self):
+        obj = mock.Mock(physical_id=None)
+        profile = dp.DockerProfile('container', self.spec)
+        self.assertFalse(profile.do_update(obj))
+
+    def test_check_container_name(self):
+        obj = mock.Mock(physical_id='FAKE_ID')
+        docker = dp.DockerProfile('container', self.spec)
+
+        new_spec = {
+            'type': 'container.dockerinc.docker',
+            'version': '1.0',
+            'properties': {
+                'context': {
+                    'region_name': 'RegionOne'
+                },
+                'name': 'new_name',
+                'image': 'hello-world',
+                'command': '/bin/sleep 30',
+                'port': 2375,
+                'host_node': 'fake_node',
+            }
+        }
+        new_profile = dp.DockerProfile('u', new_spec)
+        res, new_name = docker._check_container_name(obj, new_profile)
+
+        self.assertTrue(res)
+
+    def test_check_container_same_name(self):
+        obj = mock.Mock(physical_id='FAKE_ID')
+        docker = dp.DockerProfile('container', self.spec)
+
+        new_spec = {
+            'type': 'container.dockerinc.docker',
+            'version': '1.0',
+            'properties': {
+                'context': {
+                    'region_name': 'RegionOne'
+                },
+                'name': 'docker_container',
+                'image': 'hello-world',
+                'command': '/bin/sleep 30',
+                'port': 2375,
+                'host_node': 'fake_node',
+            }
+        }
+        new_profile = dp.DockerProfile('u', new_spec)
+        res, new_name = docker._check_container_name(obj, new_profile)
+
+        self.assertFalse(res)
+
     @mock.patch.object(dp.DockerProfile, 'docker')
     def test_handle_reboot(self, mock_docker):
         x_docker = mock.Mock()
@@ -488,7 +601,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
 
         self.assertIsNone(res)
         mock_docker.assert_called_once_with(obj)
-        x_docker.reboot.assert_called_once_with('FAKE_ID')
+        x_docker.restart.assert_called_once_with('FAKE_ID')
 
     @mock.patch.object(dp.DockerProfile, 'docker')
     def test_handle_reboot_with_timeout(self, mock_docker):
@@ -502,7 +615,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
 
         self.assertIsNone(res)
         mock_docker.assert_called_once_with(obj)
-        x_docker.reboot.assert_called_once_with('FAKE_ID', timeout=200)
+        x_docker.restart.assert_called_once_with('FAKE_ID', timeout=200)
 
     def test_handle_reboot_no_physical_id(self):
         obj = mock.Mock(physical_id=None)
@@ -529,7 +642,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
     def test_handle_reboot_docker_failure(self, mock_docker):
         x_docker = mock.Mock()
         mock_docker.return_value = x_docker
-        x_docker.reboot.side_effect = exc.InternalError(message="Boom")
+        x_docker.restart.side_effect = exc.InternalError(message="Boom")
         obj = mock.Mock(physical_id='FAKE_ID')
         docker = dp.DockerProfile('container', self.spec)
 
@@ -639,3 +752,67 @@ class TestContainerDockerProfile(base.SenlinTestCase):
 
         self.assertEqual("Failed in unpausing container 'FAKE_ID': "
                          "Boom.", six.text_type(ex))
+
+    @mock.patch.object(dp.DockerProfile, 'docker')
+    def test_handle_stop(self, mock_docker):
+        x_docker = mock.Mock()
+        x_docker = mock_docker.return_value
+        obj = mock.Mock(physical_id='FAKE_ID')
+
+        docker = dp.DockerProfile('container', self.spec)
+        params = {'timeout': None}
+        res = docker.handle_stop(obj, **params)
+
+        self.assertIsNone(res)
+        mock_docker.assert_called_once_with(obj)
+        x_docker.stop.assert_called_once_with('FAKE_ID', **params)
+
+    @mock.patch.object(dp.DockerProfile, 'docker')
+    def test_handle_stop_with_timeout(self, mock_docker):
+        x_docker = mock.Mock()
+        x_docker = mock_docker.return_value
+        obj = mock.Mock(physical_id='FAKE_ID')
+
+        docker = dp.DockerProfile('container', self.spec)
+        params = {'timeout': 200}
+        res = docker.handle_stop(obj, **params)
+
+        self.assertIsNone(res)
+        mock_docker.assert_called_once_with(obj)
+        x_docker.stop.assert_called_once_with('FAKE_ID', **params)
+
+    def test_handle_stop_no_physical_id(self):
+        obj = mock.Mock(physical_id=None)
+        docker = dp.DockerProfile('container', self.spec)
+
+        res = docker.handle_stop(obj)
+
+        self.assertIsNone(res)
+
+    @mock.patch.object(dp.DockerProfile, 'docker')
+    def test_handle_stop_driver_failure(self, mock_docker):
+        mock_docker.side_effect = exc.InternalError(message="Boom")
+        obj = mock.Mock(physical_id='FAKE_ID')
+        docker = dp.DockerProfile('container', self.spec)
+
+        ex = self.assertRaises(exc.EResourceOperation,
+                               docker.handle_stop,
+                               obj)
+
+        self.assertEqual("Failed in stop container 'FAKE_ID': "
+                         "Boom.", six.text_type(ex))
+
+    @mock.patch.object(dp.DockerProfile, 'docker')
+    def test_handle_stop_docker_failure(self, mock_docker):
+        x_docker = mock.Mock()
+        mock_docker.return_value = x_docker
+        x_docker.stop.side_effect = exc.InternalError(message="Boom")
+        obj = mock.Mock(physical_id='FAKE_ID')
+        docker = dp.DockerProfile('container', self.spec)
+
+        ex = self.assertRaises(exc.EResourceOperation,
+                               docker.handle_stop,
+                               obj)
+
+        self.assertEqual("Failed in stop container 'FAKE_ID': "
+                         "Boom.", six.text_type(ex))
