@@ -16,6 +16,7 @@ import mock
 from oslo_context import context as oslo_ctx
 import six
 
+from senlin.common import consts
 from senlin.common import context as senlin_ctx
 from senlin.common import exception
 from senlin.common import schema
@@ -84,6 +85,7 @@ class TestProfileBase(base.SenlinTestCase):
         self.ctx = utils.dummy_context(project='profile_test_project')
         g_env = environment.global_env()
         g_env.register_profile('os.dummy-1.0', DummyProfile)
+        g_env.register_profile('os.dummy-1.1', DummyProfile)
         self.spec = parser.simple_parse(sample_profile)
 
     def _create_profile(self, name, pid=None, context=None):
@@ -130,6 +132,72 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertIsNone(profile._orchestrationclient)
         self.assertIsNone(profile._block_storageclient)
 
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test_init_version_as_float(self, mock_creds):
+        mock_creds.return_value = {'foo': 'bar'}
+        self.spec['version'] = 1.1
+        profile = self._create_profile('test-profile')
+
+        self.assertIsNone(profile.id)
+        self.assertEqual('test-profile', profile.name)
+        self.assertEqual(self.spec, profile.spec)
+        self.assertEqual('os.dummy', profile.type_name)
+        self.assertEqual('1.1', profile.version)
+        self.assertEqual('os.dummy-1.1', profile.type)
+        self.assertEqual(self.ctx.user_id, profile.user)
+        self.assertEqual(self.ctx.project_id, profile.project)
+        self.assertEqual(self.ctx.domain_id, profile.domain)
+        self.assertEqual({}, profile.metadata)
+        self.assertIsNone(profile.created_at)
+        self.assertIsNone(profile.updated_at)
+
+        spec_data = profile.spec_data
+        self.assertEqual('os.dummy', spec_data['type'])
+        self.assertEqual('1.1', spec_data['version'])
+        self.assertEqual('value1', spec_data['properties']['key1'])
+        self.assertEqual(2, spec_data['properties']['key2'])
+        self.assertEqual('value1', profile.properties['key1'])
+        self.assertEqual(2, profile.properties['key2'])
+        self.assertEqual({'foo': 'bar'}, profile.context)
+
+        self.assertIsNone(profile._computeclient)
+        self.assertIsNone(profile._networkclient)
+        self.assertIsNone(profile._orchestrationclient)
+        self.assertIsNone(profile._block_storageclient)
+
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
+    def test_init_version_as_string(self, mock_creds):
+        mock_creds.return_value = {'foo': 'bar'}
+        self.spec['version'] = '1.1'
+        profile = self._create_profile('test-profile')
+
+        self.assertIsNone(profile.id)
+        self.assertEqual('test-profile', profile.name)
+        self.assertEqual(self.spec, profile.spec)
+        self.assertEqual('os.dummy', profile.type_name)
+        self.assertEqual('1.1', profile.version)
+        self.assertEqual('os.dummy-1.1', profile.type)
+        self.assertEqual(self.ctx.user_id, profile.user)
+        self.assertEqual(self.ctx.project_id, profile.project)
+        self.assertEqual(self.ctx.domain_id, profile.domain)
+        self.assertEqual({}, profile.metadata)
+        self.assertIsNone(profile.created_at)
+        self.assertIsNone(profile.updated_at)
+
+        spec_data = profile.spec_data
+        self.assertEqual('os.dummy', spec_data['type'])
+        self.assertEqual('1.1', spec_data['version'])
+        self.assertEqual('value1', spec_data['properties']['key1'])
+        self.assertEqual(2, spec_data['properties']['key2'])
+        self.assertEqual('value1', profile.properties['key1'])
+        self.assertEqual(2, profile.properties['key2'])
+        self.assertEqual({'foo': 'bar'}, profile.context)
+
+        self.assertIsNone(profile._computeclient)
+        self.assertIsNone(profile._networkclient)
+        self.assertIsNone(profile._orchestrationclient)
+        self.assertIsNone(profile._block_storageclient)
+
     @mock.patch.object(senlin_ctx, 'get_service_credentials')
     def test_init_with_context(self, mock_creds):
         mock_creds.return_value = {'foo': 'bar'}
@@ -374,10 +442,10 @@ class TestProfileBase(base.SenlinTestCase):
             side_effect=exception.InternalError(code=400, message='BAD'))
         obj = mock_load
 
-        res = profile.check_object(self.ctx, obj)
+        self.assertRaises(exception.InternalError, profile.check_object,
+                          self.ctx, obj)
 
         profile.load(self.ctx).do_check.assert_called_once_with(obj)
-        self.assertFalse(res)
 
     @mock.patch.object(pb.Profile, 'load')
     def test_update_object_with_profile(self, mock_load):
@@ -570,7 +638,7 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertRaises(exception.ESchema, profile.validate)
 
     @mock.patch.object(senlin_ctx, 'get_service_credentials')
-    def test__init_context(self, mock_creds):
+    def test_init_context(self, mock_creds):
         fake_ctx = mock.Mock()
         mock_creds.return_value = fake_ctx
 
@@ -585,7 +653,7 @@ class TestProfileBase(base.SenlinTestCase):
         mock_creds.assert_called_once_with()
 
     @mock.patch.object(senlin_ctx, 'get_service_credentials')
-    def test__init_context_for_real(self, mock_creds):
+    def test_init_context_for_real(self, mock_creds):
         fake_ctx = {
             'project_name': 'this project',
             'project_domain_name': 'this domain',
@@ -607,7 +675,7 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertEqual(expected, profile.context)
 
     @mock.patch.object(senlin_ctx, 'get_service_credentials')
-    def test__init_context_for_real_with_data(self, mock_creds):
+    def test_init_context_for_real_with_data(self, mock_creds):
         fake_ctx = {
             'project_name': 'this project',
             'project_domain_name': 'this domain',
@@ -633,7 +701,7 @@ class TestProfileBase(base.SenlinTestCase):
 
     @mock.patch.object(co.Credential, 'get')
     @mock.patch.object(oslo_ctx, 'get_current')
-    def test__build_conn_params(self, mock_current, mock_get):
+    def test_build_conn_params(self, mock_current, mock_get):
         profile = self._create_profile('test-profile')
         profile.context = {'foo': 'bar'}
         fake_cred = mock.Mock(cred={'openstack': {'trust': 'TRUST_ID'}})
@@ -655,7 +723,7 @@ class TestProfileBase(base.SenlinTestCase):
 
     @mock.patch.object(co.Credential, 'get')
     @mock.patch.object(oslo_ctx, 'get_current')
-    def test__build_conn_params_trust_not_found(self, mock_current, mock_get):
+    def test_build_conn_params_trust_not_found(self, mock_current, mock_get):
         profile = self._create_profile('test-profile')
         mock_get.return_value = None
         fake_ctx = mock.Mock()
@@ -696,6 +764,25 @@ class TestProfileBase(base.SenlinTestCase):
 
         self.assertEqual(cc, res)
 
+    @mock.patch.object(pb.Profile, '_build_conn_params')
+    @mock.patch("senlin.drivers.base.SenlinDriver")
+    def test_glance_client(self, mock_senlindriver, mock_params):
+        obj = mock.Mock()
+        sd = mock.Mock()
+        gc = mock.Mock()
+        sd.glance.return_value = gc
+        mock_senlindriver.return_value = sd
+        fake_params = mock.Mock()
+        mock_params.return_value = fake_params
+        profile = self._create_profile('test-profile')
+
+        res = profile.glance(obj)
+
+        self.assertEqual(gc, res)
+        self.assertEqual(gc, profile._glanceclient)
+        mock_params.assert_called_once_with(obj.user, obj.project)
+        sd.glance.assert_called_once_with(fake_params)
+
     @mock.patch.object(pb.Profile, '_build_conn_params')
     @mock.patch("senlin.drivers.base.SenlinDriver")
     def test_neutron_client(self, mock_senlindriver, mock_params):
@@ -751,11 +838,13 @@ class TestProfileBase(base.SenlinTestCase):
         self.patchobject(profile, 'do_create', return_value=True)
         self.patchobject(profile, 'do_delete', return_value=True)
 
-        res = profile.do_recover(mock.Mock())
-        self.assertTrue(res)
+        res, status = profile.do_recover(mock.Mock(),
+                                         operation=consts.RECOVER_RECREATE)
+        self.assertTrue(status)
 
-        res = profile.do_recover(mock.Mock(), operation=[{'name': 'bar'}])
-        self.assertFalse(res)
+        res, status = profile.do_recover(
+            mock.Mock(), operation='bar')
+        self.assertFalse(status)
 
     def test_do_recover_with_fencing(self):
         profile = self._create_profile('test-profile')
@@ -764,17 +853,77 @@ class TestProfileBase(base.SenlinTestCase):
         obj = mock.Mock()
 
         res = profile.do_recover(obj, ignore_missing=True,
-                                 params={"fence_compute": True})
+                                 params={"fence_compute": True},
+                                 operation=consts.RECOVER_RECREATE)
 
         self.assertTrue(res)
-        profile.do_delete.assert_called_once_with(obj, force=True)
+        profile.do_delete.assert_called_once_with(obj, force=False,
+                                                  timeout=None)
         profile.do_create.assert_called_once_with(obj)
 
+    def test_do_recover_with_delete_timeout(self):
+        profile = self._create_profile('test-profile')
+        self.patchobject(profile, 'do_create', return_value=True)
+        self.patchobject(profile, 'do_delete', return_value=True)
+        obj = mock.Mock()
+
+        res = profile.do_recover(obj, ignore_missing=True, delete_timeout=5,
+                                 operation=consts.RECOVER_RECREATE)
+
+        self.assertTrue(res)
+        profile.do_delete.assert_called_once_with(obj, force=False,
+                                                  timeout=5)
+        profile.do_create.assert_called_once_with(obj)
+
+    def test_do_recover_with_force_recreate(self):
+        profile = self._create_profile('test-profile')
+        self.patchobject(profile, 'do_create', return_value=True)
+        self.patchobject(profile, 'do_delete', return_value=True)
+        obj = mock.Mock()
+
+        res = profile.do_recover(obj, ignore_missing=True, force_recreate=True,
+                                 operation=consts.RECOVER_RECREATE)
+
+        self.assertTrue(res)
+        profile.do_delete.assert_called_once_with(obj, force=False,
+                                                  timeout=None)
+        profile.do_create.assert_called_once_with(obj)
+
+    def test_do_recover_with_force_recreate_failed_delete(self):
+        profile = self._create_profile('test-profile')
+        self.patchobject(profile, 'do_create', return_value=True)
+        err = exception.EResourceDeletion(type='STACK', id='ID',
+                                          message='BANG')
+        self.patchobject(profile, 'do_delete', side_effect=err)
+        obj = mock.Mock()
+
+        res = profile.do_recover(obj, ignore_missing=True, force_recreate=True,
+                                 operation=consts.RECOVER_RECREATE)
+        self.assertTrue(res)
+        profile.do_delete.assert_called_once_with(obj, force=False,
+                                                  timeout=None)
+        profile.do_create.assert_called_once_with(obj)
+
+    def test_do_recover_with_false_force_recreate_failed_delete(self):
+        profile = self._create_profile('test-profile')
+        err = exception.EResourceDeletion(type='STACK', id='ID',
+                                          message='BANG')
+        self.patchobject(profile, 'do_delete', side_effect=err)
+        operation = "RECREATE"
+
+        ex = self.assertRaises(exception.EResourceOperation,
+                               profile.do_recover,
+                               mock.Mock(id='NODE_ID'), operation=operation,
+                               force_recreate=False)
+        self.assertEqual("Failed in recovering node 'NODE_ID': "
+                         "Failed in deleting STACK 'ID': BANG.",
+                         six.text_type(ex))
+
     def test_do_recover_with_recreate_succeeded(self):
         profile = self._create_profile('test-profile')
         self.patchobject(profile, 'do_delete', return_value=True)
         self.patchobject(profile, 'do_create', return_value=True)
-        operation = [{"name": "RECREATE"}]
+        operation = "RECREATE"
         res = profile.do_recover(mock.Mock(), operation=operation)
 
         self.assertTrue(res)
@@ -784,7 +933,7 @@ class TestProfileBase(base.SenlinTestCase):
         err = exception.EResourceDeletion(type='STACK', id='ID',
                                           message='BANG')
         self.patchobject(profile, 'do_delete', side_effect=err)
-        operation = [{"name": "RECREATE"}]
+        operation = "RECREATE"
 
         ex = self.assertRaises(exception.EResourceOperation,
                                profile.do_recover,
@@ -798,7 +947,7 @@ class TestProfileBase(base.SenlinTestCase):
         self.patchobject(profile, 'do_delete', return_value=True)
         err = exception.EResourceCreation(type='STACK', message='BANNG')
         self.patchobject(profile, 'do_create', side_effect=err)
-        operation = [{"name": "RECREATE"}]
+        operation = "RECREATE"
 
         ex = self.assertRaises(exception.EResourceOperation,
                                profile.do_recover,
